import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.cis_sub_contractor import CisSubContractor
from ..models.cis_tax_status import CISTaxStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisVerificationDetails")


@attr.s(auto_attribs=True)
class CisVerificationDetails:
    """
    Attributes:
        manually_entered (Union[Unset, bool]):
        match_instead_of_verify (Union[Unset, bool]):
        number (Union[Unset, None, str]):
        date (Union[Unset, None, datetime.date]):
        tax_status (Union[Unset, CISTaxStatus]):
        verification_request (Union[Unset, None, str]): [readonly] If a Verification request has been made for this
            employee then this will show its ID
        verification_response (Union[Unset, CisSubContractor]):
    """

    manually_entered: Union[Unset, bool] = UNSET
    match_instead_of_verify: Union[Unset, bool] = UNSET
    number: Union[Unset, None, str] = UNSET
    date: Union[Unset, None, datetime.date] = UNSET
    tax_status: Union[Unset, CISTaxStatus] = UNSET
    verification_request: Union[Unset, None, str] = UNSET
    verification_response: Union[Unset, CisSubContractor] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        manually_entered = self.manually_entered
        match_instead_of_verify = self.match_instead_of_verify
        number = self.number
        date: Union[Unset, None, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat() if self.date else None

        tax_status: Union[Unset, str] = UNSET
        if not isinstance(self.tax_status, Unset):
            tax_status = self.tax_status.value

        verification_request = self.verification_request
        verification_response: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.verification_response, Unset):
            verification_response = self.verification_response.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if manually_entered is not UNSET:
            field_dict["manuallyEntered"] = manually_entered
        if match_instead_of_verify is not UNSET:
            field_dict["matchInsteadOfVerify"] = match_instead_of_verify
        if number is not UNSET:
            field_dict["number"] = number
        if date is not UNSET:
            field_dict["date"] = date
        if tax_status is not UNSET:
            field_dict["taxStatus"] = tax_status
        if verification_request is not UNSET:
            field_dict["verificationRequest"] = verification_request
        if verification_response is not UNSET:
            field_dict["verificationResponse"] = verification_response

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        manually_entered = d.pop("manuallyEntered", UNSET)

        match_instead_of_verify = d.pop("matchInsteadOfVerify", UNSET)

        number = d.pop("number", UNSET)

        _date = d.pop("date", UNSET)
        date: Union[Unset, None, datetime.date]
        if _date is None:
            date = None
        elif isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        _tax_status = d.pop("taxStatus", UNSET)
        tax_status: Union[Unset, CISTaxStatus]
        if isinstance(_tax_status, Unset):
            tax_status = UNSET
        else:
            tax_status = CISTaxStatus(_tax_status)

        verification_request = d.pop("verificationRequest", UNSET)

        _verification_response = d.pop("verificationResponse", UNSET)
        verification_response: Union[Unset, CisSubContractor]
        if isinstance(_verification_response, Unset):
            verification_response = UNSET
        else:
            verification_response = CisSubContractor.from_dict(_verification_response)

        cis_verification_details = cls(
            manually_entered=manually_entered,
            match_instead_of_verify=match_instead_of_verify,
            number=number,
            date=date,
            tax_status=tax_status,
            verification_request=verification_request,
            verification_response=verification_response,
        )

        return cis_verification_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.payroll_value_type import PayrollValueType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ValueOverride")


@attr.s(auto_attribs=True)
class ValueOverride:
    """
    Attributes:
        type (Union[Unset, PayrollValueType]):
        value (Union[Unset, float]): The value to use in place of the original value
        original_value (Union[Unset, float]): [readonly] The original value
        note (Union[Unset, None, str]): The reason given for the overrride
        attachment_order_id (Union[Unset, None, str]): The Id of the AttachmentOrder. Only relevant if the Type is set
            to AttachmentOrderDeductions
        pension_id (Union[Unset, None, str]): The Id of the associated Pension. Only included if the Code is PENSION,
            PENSIONSS or PENSIONRAS
    """

    type: Union[Unset, PayrollValueType] = UNSET
    value: Union[Unset, float] = UNSET
    original_value: Union[Unset, float] = UNSET
    note: Union[Unset, None, str] = UNSET
    attachment_order_id: Union[Unset, None, str] = UNSET
    pension_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        value = self.value
        original_value = self.original_value
        note = self.note
        attachment_order_id = self.attachment_order_id
        pension_id = self.pension_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value
        if original_value is not UNSET:
            field_dict["originalValue"] = original_value
        if note is not UNSET:
            field_dict["note"] = note
        if attachment_order_id is not UNSET:
            field_dict["attachmentOrderId"] = attachment_order_id
        if pension_id is not UNSET:
            field_dict["pensionId"] = pension_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, PayrollValueType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PayrollValueType(_type)

        value = d.pop("value", UNSET)

        original_value = d.pop("originalValue", UNSET)

        note = d.pop("note", UNSET)

        attachment_order_id = d.pop("attachmentOrderId", UNSET)

        pension_id = d.pop("pensionId", UNSET)

        value_override = cls(
            type=type,
            value=value,
            original_value=original_value,
            note=note,
            attachment_order_id=attachment_order_id,
            pension_id=pension_id,
        )

        return value_override



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DepartmentMembership")


@attr.s(auto_attribs=True)
class DepartmentMembership:
    """Used to represent an Employees membership of a Department

    Attributes:
        code (str): The Code for the Department
        color (Union[Unset, None, str]): [readonly] The Color for the Department
        title (Union[Unset, None, str]): [readonly] The Title for the Department
        primary (Union[Unset, bool]): Set to true if this is the primary Department for the Employee.
            Only one department can be set as the primary.
        weighting (Union[Unset, float]): If there is more than one Department Membership for the Employee then this
            determines the weighting to give to this membership.
            ie, if he is in two departments you might set the primary as 0.8 and the secondary as 0.2;
    """

    code: str
    color: Union[Unset, None, str] = UNSET
    title: Union[Unset, None, str] = UNSET
    primary: Union[Unset, bool] = UNSET
    weighting: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        color = self.color
        title = self.title
        primary = self.primary
        weighting = self.weighting

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "code": code,
            }
        )
        if color is not UNSET:
            field_dict["color"] = color
        if title is not UNSET:
            field_dict["title"] = title
        if primary is not UNSET:
            field_dict["primary"] = primary
        if weighting is not UNSET:
            field_dict["weighting"] = weighting

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code")

        color = d.pop("color", UNSET)

        title = d.pop("title", UNSET)

        primary = d.pop("primary", UNSET)

        weighting = d.pop("weighting", UNSET)

        department_membership = cls(
            code=code,
            color=color,
            title=title,
            primary=primary,
            weighting=weighting,
        )

        return department_membership



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="NicSummary")


@attr.s(auto_attribs=True)
class NicSummary:
    """
    Attributes:
        ni_table (Union[Unset, str]):
        as_director (Union[Unset, bool]):
        gross_earnings_for_nics (Union[Unset, float]):
        earnings_at_lel (Union[Unset, float]):
        earnings_above_lel_to_pt (Union[Unset, float]):
        earnings_above_pt_to_st (Union[Unset, float]):
        earnings_above_pt_to_uap (Union[Unset, float]):
        earnings_above_st_to_uel (Union[Unset, float]):
        earnings_above_st_to_fust (Union[Unset, float]):
        earnings_above_fust_to_uel (Union[Unset, float]):
        earnings_above_uap_to_uel (Union[Unset, float]):
        earnings_above_uel (Union[Unset, float]):
        employee_nics (Union[Unset, float]):
        employee_ni_rebate (Union[Unset, float]):
        employer_nics (Union[Unset, float]):
        employer_ni_rebate (Union[Unset, float]):
        has_values (Union[Unset, bool]):
        employee (Union[Unset, Item]):
    """

    ni_table: Union[Unset, str] = UNSET
    as_director: Union[Unset, bool] = UNSET
    gross_earnings_for_nics: Union[Unset, float] = UNSET
    earnings_at_lel: Union[Unset, float] = UNSET
    earnings_above_lel_to_pt: Union[Unset, float] = UNSET
    earnings_above_pt_to_st: Union[Unset, float] = UNSET
    earnings_above_pt_to_uap: Union[Unset, float] = UNSET
    earnings_above_st_to_uel: Union[Unset, float] = UNSET
    earnings_above_st_to_fust: Union[Unset, float] = UNSET
    earnings_above_fust_to_uel: Union[Unset, float] = UNSET
    earnings_above_uap_to_uel: Union[Unset, float] = UNSET
    earnings_above_uel: Union[Unset, float] = UNSET
    employee_nics: Union[Unset, float] = UNSET
    employee_ni_rebate: Union[Unset, float] = UNSET
    employer_nics: Union[Unset, float] = UNSET
    employer_ni_rebate: Union[Unset, float] = UNSET
    has_values: Union[Unset, bool] = UNSET
    employee: Union[Unset, Item] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ni_table = self.ni_table
        as_director = self.as_director
        gross_earnings_for_nics = self.gross_earnings_for_nics
        earnings_at_lel = self.earnings_at_lel
        earnings_above_lel_to_pt = self.earnings_above_lel_to_pt
        earnings_above_pt_to_st = self.earnings_above_pt_to_st
        earnings_above_pt_to_uap = self.earnings_above_pt_to_uap
        earnings_above_st_to_uel = self.earnings_above_st_to_uel
        earnings_above_st_to_fust = self.earnings_above_st_to_fust
        earnings_above_fust_to_uel = self.earnings_above_fust_to_uel
        earnings_above_uap_to_uel = self.earnings_above_uap_to_uel
        earnings_above_uel = self.earnings_above_uel
        employee_nics = self.employee_nics
        employee_ni_rebate = self.employee_ni_rebate
        employer_nics = self.employer_nics
        employer_ni_rebate = self.employer_ni_rebate
        has_values = self.has_values
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ni_table is not UNSET:
            field_dict["niTable"] = ni_table
        if as_director is not UNSET:
            field_dict["asDirector"] = as_director
        if gross_earnings_for_nics is not UNSET:
            field_dict["grossEarningsForNics"] = gross_earnings_for_nics
        if earnings_at_lel is not UNSET:
            field_dict["earningsAtLel"] = earnings_at_lel
        if earnings_above_lel_to_pt is not UNSET:
            field_dict["earningsAboveLelToPt"] = earnings_above_lel_to_pt
        if earnings_above_pt_to_st is not UNSET:
            field_dict["earningsAbovePtToSt"] = earnings_above_pt_to_st
        if earnings_above_pt_to_uap is not UNSET:
            field_dict["earningsAbovePtToUap"] = earnings_above_pt_to_uap
        if earnings_above_st_to_uel is not UNSET:
            field_dict["earningsAboveStToUel"] = earnings_above_st_to_uel
        if earnings_above_st_to_fust is not UNSET:
            field_dict["earningsAboveStToFust"] = earnings_above_st_to_fust
        if earnings_above_fust_to_uel is not UNSET:
            field_dict["earningsAboveFustToUel"] = earnings_above_fust_to_uel
        if earnings_above_uap_to_uel is not UNSET:
            field_dict["earningsAboveUapToUel"] = earnings_above_uap_to_uel
        if earnings_above_uel is not UNSET:
            field_dict["earningsAboveUel"] = earnings_above_uel
        if employee_nics is not UNSET:
            field_dict["employeeNics"] = employee_nics
        if employee_ni_rebate is not UNSET:
            field_dict["employeeNiRebate"] = employee_ni_rebate
        if employer_nics is not UNSET:
            field_dict["employerNics"] = employer_nics
        if employer_ni_rebate is not UNSET:
            field_dict["employerNiRebate"] = employer_ni_rebate
        if has_values is not UNSET:
            field_dict["hasValues"] = has_values
        if employee is not UNSET:
            field_dict["employee"] = employee

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ni_table = d.pop("niTable", UNSET)

        as_director = d.pop("asDirector", UNSET)

        gross_earnings_for_nics = d.pop("grossEarningsForNics", UNSET)

        earnings_at_lel = d.pop("earningsAtLel", UNSET)

        earnings_above_lel_to_pt = d.pop("earningsAboveLelToPt", UNSET)

        earnings_above_pt_to_st = d.pop("earningsAbovePtToSt", UNSET)

        earnings_above_pt_to_uap = d.pop("earningsAbovePtToUap", UNSET)

        earnings_above_st_to_uel = d.pop("earningsAboveStToUel", UNSET)

        earnings_above_st_to_fust = d.pop("earningsAboveStToFust", UNSET)

        earnings_above_fust_to_uel = d.pop("earningsAboveFustToUel", UNSET)

        earnings_above_uap_to_uel = d.pop("earningsAboveUapToUel", UNSET)

        earnings_above_uel = d.pop("earningsAboveUel", UNSET)

        employee_nics = d.pop("employeeNics", UNSET)

        employee_ni_rebate = d.pop("employeeNiRebate", UNSET)

        employer_nics = d.pop("employerNics", UNSET)

        employer_ni_rebate = d.pop("employerNiRebate", UNSET)

        has_values = d.pop("hasValues", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        nic_summary = cls(
            ni_table=ni_table,
            as_director=as_director,
            gross_earnings_for_nics=gross_earnings_for_nics,
            earnings_at_lel=earnings_at_lel,
            earnings_above_lel_to_pt=earnings_above_lel_to_pt,
            earnings_above_pt_to_st=earnings_above_pt_to_st,
            earnings_above_pt_to_uap=earnings_above_pt_to_uap,
            earnings_above_st_to_uel=earnings_above_st_to_uel,
            earnings_above_st_to_fust=earnings_above_st_to_fust,
            earnings_above_fust_to_uel=earnings_above_fust_to_uel,
            earnings_above_uap_to_uel=earnings_above_uap_to_uel,
            earnings_above_uel=earnings_above_uel,
            employee_nics=employee_nics,
            employee_ni_rebate=employee_ni_rebate,
            employer_nics=employer_nics,
            employer_ni_rebate=employer_ni_rebate,
            has_values=has_values,
            employee=employee,
        )

        return nic_summary



from enum import Enum


class AttachmentOrderType(str, Enum):
    AEOP = "AeoP"
    AEO = "Aeo"
    CSA = "Csa"
    CSA2012 = "Csa2012"
    DEO = "Deo"
    AEONP = "AeoNp"
    CCPRE92 = "CcPre92"
    CCPOST92 = "CcPost92"
    CTAEO = "Ctaeo"
    MCAEO = "Mcaeo"
    EA = "Ea"
    CMA = "Cma"
    CAO = "Cao"
    ISD = "Isd"
    EA2006 = "Ea2006"
    CAPS = "Caps"
    DEA = "Dea"
    DEAHIGHER = "DeaHigher"
    DEAFIXED = "DeaFixed"
    CTAEOWALES = "CtaeoWales"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AeExclusionCode(str, Enum):
    NOTKNOWN = "NotKnown"
    NOTAWORKER = "NotAWorker"
    NOTWORKINGINUK = "NotWorkingInUk"
    NOORDINARILYWORKINGINUK = "NoOrdinarilyWorkingInUk"
    OUTSIDEOFAGERANGE = "OutsideOfAgeRange"
    SINGLEEMPLOYEE = "SingleEmployee"
    CEASEDACTIVEMEMBERSHIPINPAST12MO = "CeasedActiveMembershipInPast12Mo"
    CEASEDACTIVEMEMBERSHIP = "CeasedActiveMembership"
    RECEIVEDWULSINPAST12MO = "ReceivedWulsInPast12Mo"
    RECEIVEDWULS = "ReceivedWuls"
    LEAVING = "Leaving"
    TAXPROTECTION = "TaxProtection"
    CISSUBCONTRACTOR = "CisSubContractor"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.fps_benefit import FpsBenefit
from ..models.fps_employee_flexible_drawdown import FpsEmployeeFlexibleDrawdown
from ..models.fps_employee_tax_code import FpsEmployeeTaxCode
from ..models.fps_employee_trivial_commutation_payment import (
    FpsEmployeeTrivialCommutationPayment,
)
from ..models.student_loan_recovered import StudentLoanRecovered
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeePayment")


@attr.s(auto_attribs=True)
class FpsEmployeePayment:
    """
    Attributes:
        bacs_hash_code (Union[Unset, None, str]):
        pay_freq (Union[Unset, None, str]):
        pmt_date (Union[Unset, None, str]):
        late_reason (Union[Unset, None, str]):
        week_no (Union[Unset, None, str]):
        month_no (Union[Unset, None, str]):
        periods_covered (Union[Unset, int]):
        aggregated_earnings (Union[Unset, None, str]):
        pmt_after_leaving (Union[Unset, None, str]):
        hours_worked (Union[Unset, None, str]):
        tax_code (Union[Unset, FpsEmployeeTaxCode]):
        taxable_pay (Union[Unset, None, str]):
        non_tax_or_nic_pmt (Union[Unset, None, str]):
        dedns_from_net_pay (Union[Unset, None, str]):
        pay_after_stat_dedns (Union[Unset, None, str]):
        benefits_taxed_via_payroll (Union[Unset, None, str]):
        class_1ani_cs_ytd (Union[Unset, None, str]):
        benefits (Union[Unset, FpsBenefit]):
        empee_pen_contribns_paid (Union[Unset, None, str]):
        items_subject_to_class_1nic (Union[Unset, None, str]):
        empee_pen_contribns_not_paid (Union[Unset, None, str]):
        student_loan_recovered (Union[Unset, StudentLoanRecovered]):
        postgrad_loan_recovered (Union[Unset, None, str]):
        tax_deducted_or_refunded (Union[Unset, None, str]):
        on_strike (Union[Unset, None, str]):
        unpaid_absence (Union[Unset, None, str]):
        smpytd (Union[Unset, None, str]):
        sppytd (Union[Unset, None, str]):
        sapytd (Union[Unset, None, str]):
        sh_ppytd (Union[Unset, None, str]):
        spbpytd (Union[Unset, None, str]):
        trivial_commutation_payment (Union[Unset, None, List[FpsEmployeeTrivialCommutationPayment]]):
        flexible_drawdown (Union[Unset, FpsEmployeeFlexibleDrawdown]):
    """

    bacs_hash_code: Union[Unset, None, str] = UNSET
    pay_freq: Union[Unset, None, str] = UNSET
    pmt_date: Union[Unset, None, str] = UNSET
    late_reason: Union[Unset, None, str] = UNSET
    week_no: Union[Unset, None, str] = UNSET
    month_no: Union[Unset, None, str] = UNSET
    periods_covered: Union[Unset, int] = UNSET
    aggregated_earnings: Union[Unset, None, str] = UNSET
    pmt_after_leaving: Union[Unset, None, str] = UNSET
    hours_worked: Union[Unset, None, str] = UNSET
    tax_code: Union[Unset, FpsEmployeeTaxCode] = UNSET
    taxable_pay: Union[Unset, None, str] = UNSET
    non_tax_or_nic_pmt: Union[Unset, None, str] = UNSET
    dedns_from_net_pay: Union[Unset, None, str] = UNSET
    pay_after_stat_dedns: Union[Unset, None, str] = UNSET
    benefits_taxed_via_payroll: Union[Unset, None, str] = UNSET
    class_1ani_cs_ytd: Union[Unset, None, str] = UNSET
    benefits: Union[Unset, FpsBenefit] = UNSET
    empee_pen_contribns_paid: Union[Unset, None, str] = UNSET
    items_subject_to_class_1nic: Union[Unset, None, str] = UNSET
    empee_pen_contribns_not_paid: Union[Unset, None, str] = UNSET
    student_loan_recovered: Union[Unset, StudentLoanRecovered] = UNSET
    postgrad_loan_recovered: Union[Unset, None, str] = UNSET
    tax_deducted_or_refunded: Union[Unset, None, str] = UNSET
    on_strike: Union[Unset, None, str] = UNSET
    unpaid_absence: Union[Unset, None, str] = UNSET
    smpytd: Union[Unset, None, str] = UNSET
    sppytd: Union[Unset, None, str] = UNSET
    sapytd: Union[Unset, None, str] = UNSET
    sh_ppytd: Union[Unset, None, str] = UNSET
    spbpytd: Union[Unset, None, str] = UNSET
    trivial_commutation_payment: Union[
        Unset, None, List[FpsEmployeeTrivialCommutationPayment]
    ] = UNSET
    flexible_drawdown: Union[Unset, FpsEmployeeFlexibleDrawdown] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        bacs_hash_code = self.bacs_hash_code
        pay_freq = self.pay_freq
        pmt_date = self.pmt_date
        late_reason = self.late_reason
        week_no = self.week_no
        month_no = self.month_no
        periods_covered = self.periods_covered
        aggregated_earnings = self.aggregated_earnings
        pmt_after_leaving = self.pmt_after_leaving
        hours_worked = self.hours_worked
        tax_code: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tax_code, Unset):
            tax_code = self.tax_code.to_dict()

        taxable_pay = self.taxable_pay
        non_tax_or_nic_pmt = self.non_tax_or_nic_pmt
        dedns_from_net_pay = self.dedns_from_net_pay
        pay_after_stat_dedns = self.pay_after_stat_dedns
        benefits_taxed_via_payroll = self.benefits_taxed_via_payroll
        class_1ani_cs_ytd = self.class_1ani_cs_ytd
        benefits: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.benefits, Unset):
            benefits = self.benefits.to_dict()

        empee_pen_contribns_paid = self.empee_pen_contribns_paid
        items_subject_to_class_1nic = self.items_subject_to_class_1nic
        empee_pen_contribns_not_paid = self.empee_pen_contribns_not_paid
        student_loan_recovered: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.student_loan_recovered, Unset):
            student_loan_recovered = self.student_loan_recovered.to_dict()

        postgrad_loan_recovered = self.postgrad_loan_recovered
        tax_deducted_or_refunded = self.tax_deducted_or_refunded
        on_strike = self.on_strike
        unpaid_absence = self.unpaid_absence
        smpytd = self.smpytd
        sppytd = self.sppytd
        sapytd = self.sapytd
        sh_ppytd = self.sh_ppytd
        spbpytd = self.spbpytd
        trivial_commutation_payment: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trivial_commutation_payment, Unset):
            if self.trivial_commutation_payment is None:
                trivial_commutation_payment = None
            else:
                trivial_commutation_payment = []
                for (
                    trivial_commutation_payment_item_data
                ) in self.trivial_commutation_payment:
                    trivial_commutation_payment_item = (
                        trivial_commutation_payment_item_data.to_dict()
                    )

                    trivial_commutation_payment.append(trivial_commutation_payment_item)

        flexible_drawdown: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.flexible_drawdown, Unset):
            flexible_drawdown = self.flexible_drawdown.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if bacs_hash_code is not UNSET:
            field_dict["bacsHashCode"] = bacs_hash_code
        if pay_freq is not UNSET:
            field_dict["payFreq"] = pay_freq
        if pmt_date is not UNSET:
            field_dict["pmtDate"] = pmt_date
        if late_reason is not UNSET:
            field_dict["lateReason"] = late_reason
        if week_no is not UNSET:
            field_dict["weekNo"] = week_no
        if month_no is not UNSET:
            field_dict["monthNo"] = month_no
        if periods_covered is not UNSET:
            field_dict["periodsCovered"] = periods_covered
        if aggregated_earnings is not UNSET:
            field_dict["aggregatedEarnings"] = aggregated_earnings
        if pmt_after_leaving is not UNSET:
            field_dict["pmtAfterLeaving"] = pmt_after_leaving
        if hours_worked is not UNSET:
            field_dict["hoursWorked"] = hours_worked
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code
        if taxable_pay is not UNSET:
            field_dict["taxablePay"] = taxable_pay
        if non_tax_or_nic_pmt is not UNSET:
            field_dict["nonTaxOrNICPmt"] = non_tax_or_nic_pmt
        if dedns_from_net_pay is not UNSET:
            field_dict["dednsFromNetPay"] = dedns_from_net_pay
        if pay_after_stat_dedns is not UNSET:
            field_dict["payAfterStatDedns"] = pay_after_stat_dedns
        if benefits_taxed_via_payroll is not UNSET:
            field_dict["benefitsTaxedViaPayroll"] = benefits_taxed_via_payroll
        if class_1ani_cs_ytd is not UNSET:
            field_dict["class1ANICsYTD"] = class_1ani_cs_ytd
        if benefits is not UNSET:
            field_dict["benefits"] = benefits
        if empee_pen_contribns_paid is not UNSET:
            field_dict["empeePenContribnsPaid"] = empee_pen_contribns_paid
        if items_subject_to_class_1nic is not UNSET:
            field_dict["itemsSubjectToClass1NIC"] = items_subject_to_class_1nic
        if empee_pen_contribns_not_paid is not UNSET:
            field_dict["empeePenContribnsNotPaid"] = empee_pen_contribns_not_paid
        if student_loan_recovered is not UNSET:
            field_dict["studentLoanRecovered"] = student_loan_recovered
        if postgrad_loan_recovered is not UNSET:
            field_dict["postgradLoanRecovered"] = postgrad_loan_recovered
        if tax_deducted_or_refunded is not UNSET:
            field_dict["taxDeductedOrRefunded"] = tax_deducted_or_refunded
        if on_strike is not UNSET:
            field_dict["onStrike"] = on_strike
        if unpaid_absence is not UNSET:
            field_dict["unpaidAbsence"] = unpaid_absence
        if smpytd is not UNSET:
            field_dict["smpytd"] = smpytd
        if sppytd is not UNSET:
            field_dict["sppytd"] = sppytd
        if sapytd is not UNSET:
            field_dict["sapytd"] = sapytd
        if sh_ppytd is not UNSET:
            field_dict["shPPYTD"] = sh_ppytd
        if spbpytd is not UNSET:
            field_dict["spbpytd"] = spbpytd
        if trivial_commutation_payment is not UNSET:
            field_dict["trivialCommutationPayment"] = trivial_commutation_payment
        if flexible_drawdown is not UNSET:
            field_dict["flexibleDrawdown"] = flexible_drawdown

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bacs_hash_code = d.pop("bacsHashCode", UNSET)

        pay_freq = d.pop("payFreq", UNSET)

        pmt_date = d.pop("pmtDate", UNSET)

        late_reason = d.pop("lateReason", UNSET)

        week_no = d.pop("weekNo", UNSET)

        month_no = d.pop("monthNo", UNSET)

        periods_covered = d.pop("periodsCovered", UNSET)

        aggregated_earnings = d.pop("aggregatedEarnings", UNSET)

        pmt_after_leaving = d.pop("pmtAfterLeaving", UNSET)

        hours_worked = d.pop("hoursWorked", UNSET)

        _tax_code = d.pop("taxCode", UNSET)
        tax_code: Union[Unset, FpsEmployeeTaxCode]
        if isinstance(_tax_code, Unset):
            tax_code = UNSET
        else:
            tax_code = FpsEmployeeTaxCode.from_dict(_tax_code)

        taxable_pay = d.pop("taxablePay", UNSET)

        non_tax_or_nic_pmt = d.pop("nonTaxOrNICPmt", UNSET)

        dedns_from_net_pay = d.pop("dednsFromNetPay", UNSET)

        pay_after_stat_dedns = d.pop("payAfterStatDedns", UNSET)

        benefits_taxed_via_payroll = d.pop("benefitsTaxedViaPayroll", UNSET)

        class_1ani_cs_ytd = d.pop("class1ANICsYTD", UNSET)

        _benefits = d.pop("benefits", UNSET)
        benefits: Union[Unset, FpsBenefit]
        if isinstance(_benefits, Unset):
            benefits = UNSET
        else:
            benefits = FpsBenefit.from_dict(_benefits)

        empee_pen_contribns_paid = d.pop("empeePenContribnsPaid", UNSET)

        items_subject_to_class_1nic = d.pop("itemsSubjectToClass1NIC", UNSET)

        empee_pen_contribns_not_paid = d.pop("empeePenContribnsNotPaid", UNSET)

        _student_loan_recovered = d.pop("studentLoanRecovered", UNSET)
        student_loan_recovered: Union[Unset, StudentLoanRecovered]
        if isinstance(_student_loan_recovered, Unset):
            student_loan_recovered = UNSET
        else:
            student_loan_recovered = StudentLoanRecovered.from_dict(
                _student_loan_recovered
            )

        postgrad_loan_recovered = d.pop("postgradLoanRecovered", UNSET)

        tax_deducted_or_refunded = d.pop("taxDeductedOrRefunded", UNSET)

        on_strike = d.pop("onStrike", UNSET)

        unpaid_absence = d.pop("unpaidAbsence", UNSET)

        smpytd = d.pop("smpytd", UNSET)

        sppytd = d.pop("sppytd", UNSET)

        sapytd = d.pop("sapytd", UNSET)

        sh_ppytd = d.pop("shPPYTD", UNSET)

        spbpytd = d.pop("spbpytd", UNSET)

        trivial_commutation_payment = []
        _trivial_commutation_payment = d.pop("trivialCommutationPayment", UNSET)
        for trivial_commutation_payment_item_data in _trivial_commutation_payment or []:
            trivial_commutation_payment_item = (
                FpsEmployeeTrivialCommutationPayment.from_dict(
                    trivial_commutation_payment_item_data
                )
            )

            trivial_commutation_payment.append(trivial_commutation_payment_item)

        _flexible_drawdown = d.pop("flexibleDrawdown", UNSET)
        flexible_drawdown: Union[Unset, FpsEmployeeFlexibleDrawdown]
        if isinstance(_flexible_drawdown, Unset):
            flexible_drawdown = UNSET
        else:
            flexible_drawdown = FpsEmployeeFlexibleDrawdown.from_dict(
                _flexible_drawdown
            )

        fps_employee_payment = cls(
            bacs_hash_code=bacs_hash_code,
            pay_freq=pay_freq,
            pmt_date=pmt_date,
            late_reason=late_reason,
            week_no=week_no,
            month_no=month_no,
            periods_covered=periods_covered,
            aggregated_earnings=aggregated_earnings,
            pmt_after_leaving=pmt_after_leaving,
            hours_worked=hours_worked,
            tax_code=tax_code,
            taxable_pay=taxable_pay,
            non_tax_or_nic_pmt=non_tax_or_nic_pmt,
            dedns_from_net_pay=dedns_from_net_pay,
            pay_after_stat_dedns=pay_after_stat_dedns,
            benefits_taxed_via_payroll=benefits_taxed_via_payroll,
            class_1ani_cs_ytd=class_1ani_cs_ytd,
            benefits=benefits,
            empee_pen_contribns_paid=empee_pen_contribns_paid,
            items_subject_to_class_1nic=items_subject_to_class_1nic,
            empee_pen_contribns_not_paid=empee_pen_contribns_not_paid,
            student_loan_recovered=student_loan_recovered,
            postgrad_loan_recovered=postgrad_loan_recovered,
            tax_deducted_or_refunded=tax_deducted_or_refunded,
            on_strike=on_strike,
            unpaid_absence=unpaid_absence,
            smpytd=smpytd,
            sppytd=sppytd,
            sapytd=sapytd,
            sh_ppytd=sh_ppytd,
            spbpytd=spbpytd,
            trivial_commutation_payment=trivial_commutation_payment,
            flexible_drawdown=flexible_drawdown,
        )

        return fps_employee_payment



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.bank_holiday_collection import BankHolidayCollection
from ..models.pro_rata_rule import ProRataRule
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkingPattern")


@attr.s(auto_attribs=True)
class WorkingPattern:
    """
    Attributes:
        title (str):
        mon (Union[Unset, float]):
        tue (Union[Unset, float]):
        wed (Union[Unset, float]):
        thu (Union[Unset, float]):
        fri (Union[Unset, float]):
        sat (Union[Unset, float]):
        sun (Union[Unset, float]):
        bank_holidays (Union[Unset, BankHolidayCollection]):
        pro_rata_rule (Union[Unset, ProRataRule]):
        bank_holiday_dates (Union[Unset, None, List[datetime.datetime]]): [readonly] The dates that are classed as Bank
            Holidays for this Working Pattern
        is_default (Union[Unset, bool]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    title: str
    mon: Union[Unset, float] = UNSET
    tue: Union[Unset, float] = UNSET
    wed: Union[Unset, float] = UNSET
    thu: Union[Unset, float] = UNSET
    fri: Union[Unset, float] = UNSET
    sat: Union[Unset, float] = UNSET
    sun: Union[Unset, float] = UNSET
    bank_holidays: Union[Unset, BankHolidayCollection] = UNSET
    pro_rata_rule: Union[Unset, ProRataRule] = UNSET
    bank_holiday_dates: Union[Unset, None, List[datetime.datetime]] = UNSET
    is_default: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        mon = self.mon
        tue = self.tue
        wed = self.wed
        thu = self.thu
        fri = self.fri
        sat = self.sat
        sun = self.sun
        bank_holidays: Union[Unset, str] = UNSET
        if not isinstance(self.bank_holidays, Unset):
            bank_holidays = self.bank_holidays.value

        pro_rata_rule: Union[Unset, str] = UNSET
        if not isinstance(self.pro_rata_rule, Unset):
            pro_rata_rule = self.pro_rata_rule.value

        bank_holiday_dates: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.bank_holiday_dates, Unset):
            if self.bank_holiday_dates is None:
                bank_holiday_dates = None
            else:
                bank_holiday_dates = []
                for bank_holiday_dates_item_data in self.bank_holiday_dates:
                    bank_holiday_dates_item = bank_holiday_dates_item_data.isoformat()

                    bank_holiday_dates.append(bank_holiday_dates_item)

        is_default = self.is_default
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "title": title,
            }
        )
        if mon is not UNSET:
            field_dict["mon"] = mon
        if tue is not UNSET:
            field_dict["tue"] = tue
        if wed is not UNSET:
            field_dict["wed"] = wed
        if thu is not UNSET:
            field_dict["thu"] = thu
        if fri is not UNSET:
            field_dict["fri"] = fri
        if sat is not UNSET:
            field_dict["sat"] = sat
        if sun is not UNSET:
            field_dict["sun"] = sun
        if bank_holidays is not UNSET:
            field_dict["bankHolidays"] = bank_holidays
        if pro_rata_rule is not UNSET:
            field_dict["proRataRule"] = pro_rata_rule
        if bank_holiday_dates is not UNSET:
            field_dict["bankHolidayDates"] = bank_holiday_dates
        if is_default is not UNSET:
            field_dict["isDefault"] = is_default
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title")

        mon = d.pop("mon", UNSET)

        tue = d.pop("tue", UNSET)

        wed = d.pop("wed", UNSET)

        thu = d.pop("thu", UNSET)

        fri = d.pop("fri", UNSET)

        sat = d.pop("sat", UNSET)

        sun = d.pop("sun", UNSET)

        _bank_holidays = d.pop("bankHolidays", UNSET)
        bank_holidays: Union[Unset, BankHolidayCollection]
        if isinstance(_bank_holidays, Unset):
            bank_holidays = UNSET
        else:
            bank_holidays = BankHolidayCollection(_bank_holidays)

        _pro_rata_rule = d.pop("proRataRule", UNSET)
        pro_rata_rule: Union[Unset, ProRataRule]
        if isinstance(_pro_rata_rule, Unset):
            pro_rata_rule = UNSET
        else:
            pro_rata_rule = ProRataRule(_pro_rata_rule)

        bank_holiday_dates = []
        _bank_holiday_dates = d.pop("bankHolidayDates", UNSET)
        for bank_holiday_dates_item_data in _bank_holiday_dates or []:
            bank_holiday_dates_item = isoparse(bank_holiday_dates_item_data)

            bank_holiday_dates.append(bank_holiday_dates_item)

        is_default = d.pop("isDefault", UNSET)

        id = d.pop("id", UNSET)

        working_pattern = cls(
            title=title,
            mon=mon,
            tue=tue,
            wed=wed,
            thu=thu,
            fri=fri,
            sat=sat,
            sun=sun,
            bank_holidays=bank_holidays,
            pro_rata_rule=pro_rata_rule,
            bank_holiday_dates=bank_holiday_dates,
            is_default=is_default,
            id=id,
        )

        return working_pattern



from enum import Enum


class BenefitDetailsNonClass1AType(str, Enum):
    OTHER = "Other"
    MULTIPLE = "Multiple"
    LOANSWRITTENORWAIVED = "LoansWrittenOrWaived"
    NURSERYPLACES = "NurseryPlaces"
    EDUCATIONALASSITANCE = "EducationalAssitance"
    SUBSCRIPTIONSANDFEES = "SubscriptionsAndFees"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/account/verify".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[Any]:
    """Re-send Verification Email

     Users should receive an email to verify their email address when they first register or if they
    change their email address.
    However, this API endpoint can be used to re-send the email.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[Any]:
    """Re-send Verification Email

     Users should receive an email to verify their email address when they first register or if they
    change their email address.
    However, this API endpoint can be used to re-send the email.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.background_task_status import BackgroundTaskStatus
from ..models.item import Item
from ..models.role import Role
from ..types import UNSET, Unset

T = TypeVar("T", bound="Invitation")


@attr.s(auto_attribs=True)
class Invitation:
    """Invitations are used to invite other users to access an existing Employer

    Attributes:
        email_address (str): The email address of the user that is being invited to access the Employer
        employer (Union[Unset, Item]):
        message (Union[Unset, None, str]): An optional message to include in the email sent to the EmailAddress
        role (Union[Unset, Role]):
        email_id (Union[Unset, None, str]):
        email_status (Union[Unset, BackgroundTaskStatus]):
        invited_by (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    email_address: str
    employer: Union[Unset, Item] = UNSET
    message: Union[Unset, None, str] = UNSET
    role: Union[Unset, Role] = UNSET
    email_id: Union[Unset, None, str] = UNSET
    email_status: Union[Unset, BackgroundTaskStatus] = UNSET
    invited_by: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        email_address = self.email_address
        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        message = self.message
        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        email_id = self.email_id
        email_status: Union[Unset, str] = UNSET
        if not isinstance(self.email_status, Unset):
            email_status = self.email_status.value

        invited_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.invited_by, Unset):
            invited_by = self.invited_by.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "emailAddress": email_address,
            }
        )
        if employer is not UNSET:
            field_dict["employer"] = employer
        if message is not UNSET:
            field_dict["message"] = message
        if role is not UNSET:
            field_dict["role"] = role
        if email_id is not UNSET:
            field_dict["emailId"] = email_id
        if email_status is not UNSET:
            field_dict["emailStatus"] = email_status
        if invited_by is not UNSET:
            field_dict["invitedBy"] = invited_by
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email_address = d.pop("emailAddress")

        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, Item]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = Item.from_dict(_employer)

        message = d.pop("message", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, Role]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = Role(_role)

        email_id = d.pop("emailId", UNSET)

        _email_status = d.pop("emailStatus", UNSET)
        email_status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_email_status, Unset):
            email_status = UNSET
        else:
            email_status = BackgroundTaskStatus(_email_status)

        _invited_by = d.pop("invitedBy", UNSET)
        invited_by: Union[Unset, Item]
        if isinstance(_invited_by, Unset):
            invited_by = UNSET
        else:
            invited_by = Item.from_dict(_invited_by)

        id = d.pop("id", UNSET)

        invitation = cls(
            email_address=email_address,
            employer=employer,
            message=message,
            role=role,
            email_id=email_id,
            email_status=email_status,
            invited_by=invited_by,
            id=id,
        )

        return invitation



from typing import Any, Dict, List, Optional

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.external_data_provider_id import ExternalDataProviderId
from ...models.item import Item
from ...types import Response


def _get_kwargs(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/external-data/{id}/employees/push".format(
        client.base_url, employerId=employer_id, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[Item]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[List[Item]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
) -> Response[List[Item]]:
    """Push Employees

     Push employees to the ExternalDataProvider.

    Args:
        employer_id (str):
        id (ExternalDataProviderId):

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
) -> Optional[List[Item]]:
    """Push Employees

     Push employees to the ExternalDataProvider.

    Args:
        employer_id (str):
        id (ExternalDataProviderId):

    Returns:
        Response[List[Item]]
    """

    return sync_detailed(
        employer_id=employer_id,
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
) -> Response[List[Item]]:
    """Push Employees

     Push employees to the ExternalDataProvider.

    Args:
        employer_id (str):
        id (ExternalDataProviderId):

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
) -> Optional[List[Item]]:
    """Push Employees

     Push employees to the ExternalDataProvider.

    Args:
        employer_id (str):
        id (ExternalDataProviderId):

    Returns:
        Response[List[Item]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            id=id,
            client=client,
        )
    ).parsed



""" Contains all the data models used in inputs/outputs """

from .add_document_attachment_order_multipart_data import (
    AddDocumentAttachmentOrderMultipartData,
)
from .add_document_leave_multipart_data import AddDocumentLeaveMultipartData
from .add_document_loan_multipart_data import AddDocumentLoanMultipartData
from .add_document_note_multipart_data import AddDocumentNoteMultipartData
from .add_document_processing_note_multipart_data import (
    AddDocumentProcessingNoteMultipartData,
)
from .address import Address
from .ae_action import AeAction
from .ae_assessment import AeAssessment
from .ae_assessment_action import AeAssessmentAction
from .ae_employee_state import AeEmployeeState
from .ae_exclusion_code import AeExclusionCode
from .ae_status import AeStatus
from .ae_statutory_letter import AeStatutoryLetter
from .ae_uk_worker import AeUKWorker
from .analysis_report import AnalysisReport
from .analysis_report_line import AnalysisReportLine
from .analysis_report_line_value import AnalysisReportLineValue
from .analysis_report_report_response import AnalysisReportReportResponse
from .apply_year_end_employer_json_body import ApplyYearEndEmployerJsonBody
from .attachment_order import AttachmentOrder
from .attachment_order_payment import AttachmentOrderPayment
from .attachment_order_report_response import AttachmentOrderReportResponse
from .attachment_order_type import AttachmentOrderType
from .audit_event_action import AuditEventAction
from .auth_scheme import AuthScheme
from .auto_enrolment import AutoEnrolment
from .auto_enrolment_settings import AutoEnrolmentSettings
from .auto_pilot_finalise_time import AutoPilotFinaliseTime
from .automation_settings import AutomationSettings
from .average_weekly_earnings import AverageWeeklyEarnings
from .average_weekly_earnings_result import AverageWeeklyEarningsResult
from .background_task_status import BackgroundTaskStatus
from .bank_details import BankDetails
from .bank_holiday_collection import BankHolidayCollection
from .bank_payment_instruction import BankPaymentInstruction
from .bank_payment_instruction_report_response import (
    BankPaymentInstructionReportResponse,
)
from .bank_payment_instructions_csv_format import BankPaymentInstructionsCsvFormat
from .benefit import Benefit
from .benefit_declaration_type import BenefitDeclarationType
from .benefit_details_asset_type import BenefitDetailsAssetType
from .benefit_details_car import BenefitDetailsCar
from .benefit_details_car_power_type import BenefitDetailsCarPowerType
from .benefit_details_class_1a_type import BenefitDetailsClass1AType
from .benefit_details_loan import BenefitDetailsLoan
from .benefit_details_non_class_1a_type import BenefitDetailsNonClass1AType
from .benefit_details_payment_type import BenefitDetailsPaymentType
from .benefit_details_use_of_asset_type import BenefitDetailsUseOfAssetType
from .benefit_type import BenefitType
from .bureau_settings import BureauSettings
from .calendar_entry import CalendarEntry
from .calendar_entry_type import CalendarEntryType
from .car_charge import CarCharge
from .car_charge_rate import CarChargeRate
from .change_summary import ChangeSummary
from .cis_300 import Cis300
from .cis_300_declarations import Cis300Declarations
from .cis_300_report_response import Cis300ReportResponse
from .cis_contractor import CisContractor
from .cis_details import CisDetails
from .cis_partnership import CisPartnership
from .cis_request import CisRequest
from .cis_return import CisReturn
from .cis_statement import CisStatement
from .cis_statement_list_report_response import CisStatementListReportResponse
from .cis_sub_contractor import CisSubContractor
from .cis_sub_contractor_summary import CisSubContractorSummary
from .cis_sub_contractor_summary_list_report_response import (
    CisSubContractorSummaryListReportResponse,
)
from .cis_sub_contractor_type import CISSubContractorType
from .cis_tax_status import CISTaxStatus
from .cis_verification import CisVerification
from .cis_verification_details import CisVerificationDetails
from .cookie_preference import CookiePreference
from .cost_analysis_report import CostAnalysisReport
from .cost_analysis_report_report_response import CostAnalysisReportReportResponse
from .cost_breakdown import CostBreakdown
from .cost_breakdown_breakdown import CostBreakdownBreakdown
from .cost_of_employment_report import CostOfEmploymentReport
from .cost_of_employment_report_line import CostOfEmploymentReportLine
from .cost_of_employment_report_report_response import (
    CostOfEmploymentReportReportResponse,
)
from .country import Country
from .csv_file_format import CsvFileFormat
from .deferal_period_type import DeferalPeriodType
from .department import Department
from .department_membership import DepartmentMembership
from .direct_debit_mandate import DirectDebitMandate
from .directorship_details import DirectorshipDetails
from .dps_data_type import DpsDataType
from .dps_notice import DpsNotice
from .dps_settings import DpsSettings
from .email_attachment import EmailAttachment
from .emp_refs import EmpRefs
from .employee import Employee
from .employee_benefits_report import EmployeeBenefitsReport
from .employee_benefits_report_report_response import (
    EmployeeBenefitsReportReportResponse,
)
from .employee_status import EmployeeStatus
from .employee_ytd_values import EmployeeYtdValues
from .employer import Employer
from .employer_defaults import EmployerDefaults
from .employer_email import EmployerEmail
from .employer_group import EmployerGroup
from .employer_group_membership import EmployerGroupMembership
from .employer_item import EmployerItem
from .employer_opening_balances import EmployerOpeningBalances
from .employer_settings import EmployerSettings
from .employer_template import EmployerTemplate
from .employer_template_type import EmployerTemplateType
from .employment_details import EmploymentDetails
from .entity_type import EntityType
from .eps import Eps
from .eps_account import EpsAccount
from .eps_apprenticeship_levy import EpsApprenticeshipLevy
from .eps_de_minimis_state_aid import EpsDeMinimisStateAid
from .eps_final_submission import EpsFinalSubmission
from .exb import Exb
from .exb_declarations import ExbDeclarations
from .exb_employer import ExbEmployer
from .exb_p11_db import ExbP11Db
from .exb_p11_db_class_1a import ExbP11DbClass1A
from .exb_p11_db_class_1a_adjustment import ExbP11DbClass1AAdjustment
from .exb_p11_db_class_1a_adjustments import ExbP11DbClass1AAdjustments
from .exb_p11_db_class_1a_total_benefit import ExbP11DbClass1ATotalBenefit
from .exb_p11d import ExbP11D
from .exb_p11d_employee import ExbP11DEmployee
from .exb_report_response import ExbReportResponse
from .expenses_and_benefits import ExpensesAndBenefits
from .external_data_company import ExternalDataCompany
from .external_data_provider import ExternalDataProvider
from .external_data_provider_id import ExternalDataProviderId
from .external_data_provider_type import ExternalDataProviderType
from .external_employee_mapping import ExternalEmployeeMapping
from .external_employee_mapping_status import ExternalEmployeeMappingStatus
from .external_provider_conversation import ExternalProviderConversation
from .field_modification import FieldModification
from .finalise_pay_run_pay_run_json_body import FinalisePayRunPayRunJsonBody
from .fixed_code import FixedCode
from .fps import Fps
from .fps_benefit import FpsBenefit
from .fps_car import FpsCar
from .fps_car_fuel import FpsCarFuel
from .fps_employee import FpsEmployee
from .fps_employee_details import FpsEmployeeDetails
from .fps_employee_details_partner_details import FpsEmployeeDetailsPartnerDetails
from .fps_employee_figs_to_date import FpsEmployeeFigsToDate
from .fps_employee_flexible_drawdown import FpsEmployeeFlexibleDrawdown
from .fps_employee_n_iletters_and_values import FpsEmployeeNIlettersAndValues
from .fps_employee_payment import FpsEmployeePayment
from .fps_employee_starter import FpsEmployeeStarter
from .fps_employee_starter_occ_pension import FpsEmployeeStarterOccPension
from .fps_employee_tax_code import FpsEmployeeTaxCode
from .fps_employee_trivial_commutation_payment import (
    FpsEmployeeTrivialCommutationPayment,
)
from .fps_employer_pay_id_changed import FpsEmployerPayIdChanged
from .fps_employment import FpsEmployment
from .fps_fields import FpsFields
from .fps_late_reason import FpsLateReason
from .fps_report_response import FpsReportResponse
from .from_to_dates import FromToDates
from .full_payment_submission import FullPaymentSubmission
from .full_summary_of_pay_report import FullSummaryOfPayReport
from .full_summary_of_pay_report_line import FullSummaryOfPayReportLine
from .full_summary_of_pay_report_report_response import (
    FullSummaryOfPayReportReportResponse,
)
from .furlough_calculation_basis import FurloughCalculationBasis
from .furlough_report import FurloughReport
from .furlough_report_line import FurloughReportLine
from .furlough_report_report_response import FurloughReportReportResponse
from .gender import Gender
from .gov_talk import GovTalk
from .gov_talk_error import GovTalkError
from .gov_talk_submission import GovTalkSubmission
from .gross_to_net_report import GrossToNetReport
from .gross_to_net_report_cis_line import GrossToNetReportCisLine
from .gross_to_net_report_line import GrossToNetReportLine
from .gross_to_net_report_report_response import GrossToNetReportReportResponse
from .hmrc_details import HmrcDetails
from .hmrc_liability import HmrcLiability
from .hmrc_payment import HmrcPayment
from .holiday_report import HolidayReport
from .holiday_report_line import HolidayReportLine
from .holiday_report_report_response import HolidayReportReportResponse
from .holiday_type import HolidayType
from .hourly_pay_report import HourlyPayReport
from .hourly_pay_report_line import HourlyPayReportLine
from .hourly_pay_report_report_response import HourlyPayReportReportResponse
from .hours_normally_worked import HoursNormallyWorked
from .html_email_settings import HtmlEmailSettings
from .html_insertion_point import HtmlInsertionPoint
from .import_csv_employee_multipart_data import ImportCsvEmployeeMultipartData
from .import_csv_employer_multipart_data import ImportCsvEmployerMultipartData
from .invitation import Invitation
from .item import Item
from .item_list_report_response import ItemListReportResponse
from .leave import Leave
from .leave_pay_type import LeavePayType
from .leave_settings import LeaveSettings
from .leave_type import LeaveType
from .leaver_details import LeaverDetails
from .linked_piw import LinkedPiw
from .linked_piw_result import LinkedPiwResult
from .loan import Loan
from .loan_charge import LoanCharge
from .log_entry import LogEntry
from .mail_settings import MailSettings
from .marital_status import MaritalStatus
from .mileage_allowance_payments_rate import MileageAllowancePaymentsRate
from .mileage_vehicle_type import MileageVehicleType
from .monthly_minimum import MonthlyMinimum
from .national_insurance_calculation import NationalInsuranceCalculation
from .national_insurance_calculation_base import NationalInsuranceCalculationBase
from .national_insurance_calculation_period_values import (
    NationalInsuranceCalculationPeriodValues,
)
from .national_insurance_code import NationalInsuranceCode
from .national_minimum_wage import NationalMinimumWage
from .ni_letter_error import NiLetterError
from .ni_letter_validation_report import NiLetterValidationReport
from .ni_letter_validation_report_line import NiLetterValidationReportLine
from .ni_letter_validation_report_report_response import (
    NiLetterValidationReportReportResponse,
)
from .nic_summary import NicSummary
from .nominal_code_mapping import NominalCodeMapping
from .note import Note
from .note_type import NoteType
from .nvr import Nvr
from .nvr_employee import NvrEmployee
from .nvr_employee_details import NvrEmployeeDetails
from .nvr_employment import NvrEmployment
from .nvr_request import NvrRequest
from .opening_balances import OpeningBalances
from .overseas_employer_details import OverseasEmployerDetails
from .overseas_secondment_status import OverseasSecondmentStatus
from .p11 import P11
from .p11_detailed import P11Detailed
from .p11_detailed_ni_values import P11DetailedNiValues
from .p11_detailed_report_response import P11DetailedReportResponse
from .p11_line import P11Line
from .p11_ni_and_stat_payments_line import P11NiAndStatPaymentsLine
from .p11_ni_and_stat_payments_totals_line import P11NiAndStatPaymentsTotalsLine
from .p11_ni_values import P11NiValues
from .p11_paye_line import P11PayeLine
from .p11_paye_summary import P11PayeSummary
from .p11_paye_totals_line import P11PayeTotalsLine
from .p11_report_response import P11ReportResponse
from .p11d_asset_available import P11DAssetAvailable
from .p11d_asset_available_collection import P11DAssetAvailableCollection
from .p11d_asset_transferred_collection import P11DAssetTransferredCollection
from .p11d_car import P11DCar
from .p11d_car_collection import P11DCarCollection
from .p11d_car_free_fuel_withdrawn import P11DCarFreeFuelWithdrawn
from .p11d_desc_other import P11DDescOther
from .p11d_expenses import P11DExpenses
from .p11d_loan import P11DLoan
from .p11d_loan_collection import P11DLoanCollection
from .p11d_other import P11DOther
from .p11d_payment_collection import P11DPaymentCollection
from .p11d_single_item import P11DSingleItem
from .p11d_vans import P11DVans
from .p32 import P32
from .p32_report_response import P32ReportResponse
from .papdis_document import PapdisDocument
from .papdis_document_report_response import PapdisDocumentReportResponse
from .papdis_employee import PapdisEmployee
from .papdis_employee_assessment import PapdisEmployeeAssessment
from .papdis_employee_contact import PapdisEmployeeContact
from .papdis_employee_contact_postal_address import PapdisEmployeeContactPostalAddress
from .papdis_employee_contribution import PapdisEmployeeContribution
from .papdis_employee_exit import PapdisEmployeeExit
from .papdis_employee_identity import PapdisEmployeeIdentity
from .papdis_employee_name import PapdisEmployeeName
from .papdis_employee_pay import PapdisEmployeePay
from .papdis_employer import PapdisEmployer
from .papdis_message_function_code import PapdisMessageFunctionCode
from .papdis_payroll_period import PapdisPayrollPeriod
from .papdis_pension_provider import PapdisPensionProvider
from .papdis_version import PapdisVersion
from .partner_details import PartnerDetails
from .pay_basis import PayBasis
from .pay_code import PayCode
from .pay_code_calculation_type import PayCodeCalculationType
from .pay_code_multiplier_type import PayCodeMultiplierType
from .pay_line import PayLine
from .pay_method import PayMethod
from .pay_options import PayOptions
from .pay_options_import import PayOptionsImport
from .pay_period_event_type import PayPeriodEventType
from .pay_periods import PayPeriods
from .pay_run import PayRun
from .pay_run_changes import PayRunChanges
from .pay_run_cost_summary import PayRunCostSummary
from .pay_run_csv_type import PayRunCsvType
from .pay_run_entry import PayRunEntry
from .pay_run_journal import PayRunJournal
from .pay_run_payment import PayRunPayment
from .pay_run_state import PayRunState
from .pay_run_state_change import PayRunStateChange
from .pay_run_state_change_reason import PayRunStateChangeReason
from .pay_run_summary_line import PayRunSummaryLine
from .pay_run_summary_line_i_enumerable_report_response import (
    PayRunSummaryLineIEnumerableReportResponse,
)
from .pay_run_totals import PayRunTotals
from .pay_schedule import PaySchedule
from .pay_schedule_period import PaySchedulePeriod
from .pay_schedule_period_event import PaySchedulePeriodEvent
from .pay_schedule_period_events_config import PaySchedulePeriodEventsConfig
from .payee import Payee
from .payee_type import PayeeType
from .payment_date_type import PaymentDateType
from .payments_csv_mapping import PaymentsCsvMapping
from .payments_csv_mapping_column import PaymentsCsvMappingColumn
from .payments_csv_mapping_type import PaymentsCsvMappingType
from .payroll_value_type import PayrollValueType
from .payrun_email import PayrunEmail
from .payrun_payments_report import PayrunPaymentsReport
from .payrun_payments_report_report_response import PayrunPaymentsReportReportResponse
from .payslip import Payslip
from .payslip_customisation import PayslipCustomisation
from .payslip_line import PayslipLine
from .payslip_line_type import PayslipLineType
from .payslip_report_response import PayslipReportResponse
from .pdf_paper_margins import PdfPaperMargins
from .pdf_paper_orientation import PdfPaperOrientation
from .pdf_paper_size import PdfPaperSize
from .pdf_password_type import PdfPasswordType
from .pension import Pension
from .pension_administrator import PensionAdministrator
from .pension_contribution_level_type import PensionContributionLevelType
from .pension_contributions_submission import PensionContributionsSubmission
from .pension_csv_format import PensionCsvFormat
from .pension_provider import PensionProvider
from .pension_rule import PensionRule
from .pension_scheme import PensionScheme
from .pension_selection import PensionSelection
from .pension_summary import PensionSummary
from .pensioner_payroll import PensionerPayroll
from .personal_details import PersonalDetails
from .post import Post
from .pro_rata_rule import ProRataRule
from .processing_note import ProcessingNote
from .recoverable_amounts import RecoverableAmounts
from .report import Report
from .report_custom_css_option import ReportCustomCssOption
from .report_pack import ReportPack
from .report_response import ReportResponse
from .right_to_work import RightToWork
from .right_to_work_document_type import RightToWorkDocumentType
from .right_to_work_report import RightToWorkReport
from .right_to_work_report_line import RightToWorkReportLine
from .right_to_work_report_report_response import RightToWorkReportReportResponse
from .role import Role
from .rti_agent import RtiAgent
from .rti_contact import RtiContact
from .rti_employee_address import RtiEmployeeAddress
from .rti_employee_name import RtiEmployeeName
from .rti_sender_type import RtiSenderType
from .rti_submission_settings import RtiSubmissionSettings
from .rti_validation_warning import RtiValidationWarning
from .rti_validation_warning_type import RtiValidationWarningType
from .smtp_encryption import SmtpEncryption
from .smtp_settings import SmtpSettings
from .starter_declaration import StarterDeclaration
from .starter_details import StarterDetails
from .statutory_pay import StatutoryPay
from .statutory_pay_report import StatutoryPayReport
from .statutory_pay_report_line import StatutoryPayReportLine
from .statutory_pay_report_report_response import StatutoryPayReportReportResponse
from .string_string_key_value_pair import StringStringKeyValuePair
from .student_loan import StudentLoan
from .student_loan_recovered import StudentLoanRecovered
from .submission_status import SubmissionStatus
from .tag import Tag
from .tax_and_ni import TaxAndNi
from .tax_bracket import TaxBracket
from .tax_code_change import TaxCodeChange
from .tax_code_change_report import TaxCodeChangeReport
from .tax_code_change_report_report_response import TaxCodeChangeReportReportResponse
from .tax_code_change_values import TaxCodeChangeValues
from .tax_year import TaxYear
from .teachers_pension_details import TeachersPensionDetails
from .teachers_pension_employment_type import TeachersPensionEmploymentType
from .tenant import Tenant
from .tenant_billing_settings import TenantBillingSettings
from .tenant_email import TenantEmail
from .tenant_html_insertion import TenantHtmlInsertion
from .tenant_item import TenantItem
from .tiered_pension import TieredPension
from .tiered_pension_rate import TieredPensionRate
from .umbrella_payment import UmbrellaPayment
from .umbrella_reconciliation_report import UmbrellaReconciliationReport
from .umbrella_reconciliation_report_line import UmbrellaReconciliationReportLine
from .umbrella_reconciliation_report_report_response import (
    UmbrellaReconciliationReportReportResponse,
)
from .umbrella_settings import UmbrellaSettings
from .update_fav_icon_tenant_multipart_data import UpdateFavIconTenantMultipartData
from .update_logo_employer_multipart_data import UpdateLogoEmployerMultipartData
from .update_logo_tenant_multipart_data import UpdateLogoTenantMultipartData
from .update_photo_account_multipart_data import UpdatePhotoAccountMultipartData
from .update_photo_employee_multipart_data import UpdatePhotoEmployeeMultipartData
from .usage_bill import UsageBill
from .user import User
from .user_authorization import UserAuthorization
from .user_category import UserCategory
from .user_display_preferences import UserDisplayPreferences
from .user_industry import UserIndustry
from .user_job_type import UserJobType
from .userstart_page import UserstartPage
from .utm_info import UtmInfo
from .value_override import ValueOverride
from .variance_report import VarianceReport
from .variance_report_report_response import VarianceReportReportResponse
from .veteran_details import VeteranDetails
from .webhook import Webhook
from .webhook_event import WebhookEvent
from .webhook_payload import WebhookPayload
from .worker_group import WorkerGroup
from .working_pattern import WorkingPattern
from .year_end import YearEnd
from .year_end_tax_code_change import YearEndTaxCodeChange
from .ytd_pay_run_multipart_data import YtdPayRunMultipartData
from .ytd_report import YtdReport
from .ytd_report_report_response import YtdReportReportResponse
from .ytd_value import YtdValue



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.rti_employee_name import RtiEmployeeName
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeDetailsPartnerDetails")


@attr.s(auto_attribs=True)
class FpsEmployeeDetailsPartnerDetails:
    """
    Attributes:
        nino (Union[Unset, None, str]):
        name (Union[Unset, RtiEmployeeName]):
    """

    nino: Union[Unset, None, str] = UNSET
    name: Union[Unset, RtiEmployeeName] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        nino = self.nino
        name: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.name, Unset):
            name = self.name.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if nino is not UNSET:
            field_dict["nino"] = nino
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        nino = d.pop("nino", UNSET)

        _name = d.pop("name", UNSET)
        name: Union[Unset, RtiEmployeeName]
        if isinstance(_name, Unset):
            name = UNSET
        else:
            name = RtiEmployeeName.from_dict(_name)

        fps_employee_details_partner_details = cls(
            nino=nino,
            name=name,
        )

        return fps_employee_details_partner_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.smtp_encryption import SmtpEncryption
from ..types import UNSET, Unset

T = TypeVar("T", bound="SmtpSettings")


@attr.s(auto_attribs=True)
class SmtpSettings:
    """
    Attributes:
        smtp_server (Union[Unset, None, str]):
        smtp_port (Union[Unset, int]):
        encryption (Union[Unset, SmtpEncryption]):
        smtp_username (Union[Unset, None, str]):
        smtp_password (Union[Unset, None, str]):
    """

    smtp_server: Union[Unset, None, str] = UNSET
    smtp_port: Union[Unset, int] = UNSET
    encryption: Union[Unset, SmtpEncryption] = UNSET
    smtp_username: Union[Unset, None, str] = UNSET
    smtp_password: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        smtp_server = self.smtp_server
        smtp_port = self.smtp_port
        encryption: Union[Unset, str] = UNSET
        if not isinstance(self.encryption, Unset):
            encryption = self.encryption.value

        smtp_username = self.smtp_username
        smtp_password = self.smtp_password

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if smtp_server is not UNSET:
            field_dict["smtpServer"] = smtp_server
        if smtp_port is not UNSET:
            field_dict["smtpPort"] = smtp_port
        if encryption is not UNSET:
            field_dict["encryption"] = encryption
        if smtp_username is not UNSET:
            field_dict["smtpUsername"] = smtp_username
        if smtp_password is not UNSET:
            field_dict["smtpPassword"] = smtp_password

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        smtp_server = d.pop("smtpServer", UNSET)

        smtp_port = d.pop("smtpPort", UNSET)

        _encryption = d.pop("encryption", UNSET)
        encryption: Union[Unset, SmtpEncryption]
        if isinstance(_encryption, Unset):
            encryption = UNSET
        else:
            encryption = SmtpEncryption(_encryption)

        smtp_username = d.pop("smtpUsername", UNSET)

        smtp_password = d.pop("smtpPassword", UNSET)

        smtp_settings = cls(
            smtp_server=smtp_server,
            smtp_port=smtp_port,
            encryption=encryption,
            smtp_username=smtp_username,
            smtp_password=smtp_password,
        )

        return smtp_settings



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.background_task_status import BackgroundTaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="WebhookPayload")


@attr.s(auto_attribs=True)
class WebhookPayload:
    """
    Attributes:
        url (str): [readonly] The Url that payload will be sent to
        name (Union[Unset, None, str]): [readonly] A descriptive name for this payload
        attempts (Union[Unset, int]): [readonly]  The number of attempts that have been made to deliver this payload
        status_code (Union[Unset, int]): [readonly] The status code received from the Url
        task_status (Union[Unset, BackgroundTaskStatus]):
        status_message (Union[Unset, None, str]): [readonly]  A message to accompany the status
        created (Union[Unset, datetime.date]): [readonly]  The date and time this payload was created
        payload (Union[Unset, Any]): [readonly] the JSON payload that will be sent to the URl
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    url: str
    name: Union[Unset, None, str] = UNSET
    attempts: Union[Unset, int] = UNSET
    status_code: Union[Unset, int] = UNSET
    task_status: Union[Unset, BackgroundTaskStatus] = UNSET
    status_message: Union[Unset, None, str] = UNSET
    created: Union[Unset, datetime.date] = UNSET
    payload: Union[Unset, Any] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        name = self.name
        attempts = self.attempts
        status_code = self.status_code
        task_status: Union[Unset, str] = UNSET
        if not isinstance(self.task_status, Unset):
            task_status = self.task_status.value

        status_message = self.status_message
        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        payload = self.payload
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "url": url,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if attempts is not UNSET:
            field_dict["attempts"] = attempts
        if status_code is not UNSET:
            field_dict["statusCode"] = status_code
        if task_status is not UNSET:
            field_dict["taskStatus"] = task_status
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message
        if created is not UNSET:
            field_dict["created"] = created
        if payload is not UNSET:
            field_dict["payload"] = payload
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        name = d.pop("name", UNSET)

        attempts = d.pop("attempts", UNSET)

        status_code = d.pop("statusCode", UNSET)

        _task_status = d.pop("taskStatus", UNSET)
        task_status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_task_status, Unset):
            task_status = UNSET
        else:
            task_status = BackgroundTaskStatus(_task_status)

        status_message = d.pop("statusMessage", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.date]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created).date()

        payload = d.pop("payload", UNSET)

        id = d.pop("id", UNSET)

        webhook_payload = cls(
            url=url,
            name=name,
            attempts=attempts,
            status_code=status_code,
            task_status=task_status,
            status_message=status_message,
            created=created,
            payload=payload,
            id=id,
        )

        return webhook_payload



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.pay_period_event_type import PayPeriodEventType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PaySchedulePeriodEvent")


@attr.s(auto_attribs=True)
class PaySchedulePeriodEvent:
    """An Event within a PaySchedulePeriod. The event could be scheduled x days before PaymentDate.

    Attributes:
        pay_period_event_type (Union[Unset, PayPeriodEventType]): Different events supported for a PaySchedule Period.
            These events happen a pre-configured number of days before the PaymentDate for that PaySchedulePeriod.
        event_date (Union[Unset, datetime.date]): The expected date and time of the event.
        actual_event_date (Union[Unset, None, datetime.date]): The actual date and time when the event was completed.
        is_over_due (Union[Unset, bool]): Returns true if its past the event date.
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    pay_period_event_type: Union[Unset, PayPeriodEventType] = UNSET
    event_date: Union[Unset, datetime.date] = UNSET
    actual_event_date: Union[Unset, None, datetime.date] = UNSET
    is_over_due: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay_period_event_type: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period_event_type, Unset):
            pay_period_event_type = self.pay_period_event_type.value

        event_date: Union[Unset, str] = UNSET
        if not isinstance(self.event_date, Unset):
            event_date = self.event_date.isoformat()

        actual_event_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.actual_event_date, Unset):
            actual_event_date = (
                self.actual_event_date.isoformat() if self.actual_event_date else None
            )

        is_over_due = self.is_over_due
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay_period_event_type is not UNSET:
            field_dict["payPeriodEventType"] = pay_period_event_type
        if event_date is not UNSET:
            field_dict["eventDate"] = event_date
        if actual_event_date is not UNSET:
            field_dict["actualEventDate"] = actual_event_date
        if is_over_due is not UNSET:
            field_dict["isOverDue"] = is_over_due
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _pay_period_event_type = d.pop("payPeriodEventType", UNSET)
        pay_period_event_type: Union[Unset, PayPeriodEventType]
        if isinstance(_pay_period_event_type, Unset):
            pay_period_event_type = UNSET
        else:
            pay_period_event_type = PayPeriodEventType(_pay_period_event_type)

        _event_date = d.pop("eventDate", UNSET)
        event_date: Union[Unset, datetime.date]
        if isinstance(_event_date, Unset):
            event_date = UNSET
        else:
            event_date = isoparse(_event_date).date()

        _actual_event_date = d.pop("actualEventDate", UNSET)
        actual_event_date: Union[Unset, None, datetime.date]
        if _actual_event_date is None:
            actual_event_date = None
        elif isinstance(_actual_event_date, Unset):
            actual_event_date = UNSET
        else:
            actual_event_date = isoparse(_actual_event_date).date()

        is_over_due = d.pop("isOverDue", UNSET)

        id = d.pop("id", UNSET)

        pay_schedule_period_event = cls(
            pay_period_event_type=pay_period_event_type,
            event_date=event_date,
            actual_event_date=actual_event_date,
            is_over_due=is_over_due,
            id=id,
        )

        return pay_schedule_period_event



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.fps_employee_starter_occ_pension import FpsEmployeeStarterOccPension
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeStarter")


@attr.s(auto_attribs=True)
class FpsEmployeeStarter:
    """
    Attributes:
        start_date (Union[Unset, None, str]):
        start_dec (Union[Unset, None, str]):
        student_loan (Union[Unset, None, str]):
        postgrad_loan (Union[Unset, None, str]):
        occ_pension (Union[Unset, FpsEmployeeStarterOccPension]):
    """

    start_date: Union[Unset, None, str] = UNSET
    start_dec: Union[Unset, None, str] = UNSET
    student_loan: Union[Unset, None, str] = UNSET
    postgrad_loan: Union[Unset, None, str] = UNSET
    occ_pension: Union[Unset, FpsEmployeeStarterOccPension] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        start_date = self.start_date
        start_dec = self.start_dec
        student_loan = self.student_loan
        postgrad_loan = self.postgrad_loan
        occ_pension: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.occ_pension, Unset):
            occ_pension = self.occ_pension.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if start_dec is not UNSET:
            field_dict["startDec"] = start_dec
        if student_loan is not UNSET:
            field_dict["studentLoan"] = student_loan
        if postgrad_loan is not UNSET:
            field_dict["postgradLoan"] = postgrad_loan
        if occ_pension is not UNSET:
            field_dict["occPension"] = occ_pension

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_date = d.pop("startDate", UNSET)

        start_dec = d.pop("startDec", UNSET)

        student_loan = d.pop("studentLoan", UNSET)

        postgrad_loan = d.pop("postgradLoan", UNSET)

        _occ_pension = d.pop("occPension", UNSET)
        occ_pension: Union[Unset, FpsEmployeeStarterOccPension]
        if isinstance(_occ_pension, Unset):
            occ_pension = UNSET
        else:
            occ_pension = FpsEmployeeStarterOccPension.from_dict(_occ_pension)

        fps_employee_starter = cls(
            start_date=start_date,
            start_dec=start_dec,
            student_loan=student_loan,
            postgrad_loan=postgrad_loan,
            occ_pension=occ_pension,
        )

        return fps_employee_starter



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.exb_p11d_employee import ExbP11DEmployee
from ..models.p11d_asset_available_collection import P11DAssetAvailableCollection
from ..models.p11d_asset_transferred_collection import P11DAssetTransferredCollection
from ..models.p11d_car_collection import P11DCarCollection
from ..models.p11d_expenses import P11DExpenses
from ..models.p11d_loan_collection import P11DLoanCollection
from ..models.p11d_other import P11DOther
from ..models.p11d_payment_collection import P11DPaymentCollection
from ..models.p11d_single_item import P11DSingleItem
from ..models.p11d_vans import P11DVans
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExbP11D")


@attr.s(auto_attribs=True)
class ExbP11D:
    """
    Attributes:
        employee (Union[Unset, ExbP11DEmployee]):
        transferred (Union[Unset, P11DAssetTransferredCollection]):
        payments (Union[Unset, P11DPaymentCollection]):
        vouchers_or_c_cs (Union[Unset, P11DSingleItem]):
        living_accom (Union[Unset, P11DSingleItem]):
        mileage_allow (Union[Unset, P11DSingleItem]):
        cars (Union[Unset, P11DCarCollection]):
        vans (Union[Unset, P11DVans]):
        loans (Union[Unset, P11DLoanCollection]):
        medical (Union[Unset, P11DSingleItem]):
        relocation (Union[Unset, P11DSingleItem]):
        services (Union[Unset, P11DSingleItem]):
        assets_avail (Union[Unset, P11DAssetAvailableCollection]):
        other (Union[Unset, P11DOther]):
        exp_paid (Union[Unset, P11DExpenses]):
    """

    employee: Union[Unset, ExbP11DEmployee] = UNSET
    transferred: Union[Unset, P11DAssetTransferredCollection] = UNSET
    payments: Union[Unset, P11DPaymentCollection] = UNSET
    vouchers_or_c_cs: Union[Unset, P11DSingleItem] = UNSET
    living_accom: Union[Unset, P11DSingleItem] = UNSET
    mileage_allow: Union[Unset, P11DSingleItem] = UNSET
    cars: Union[Unset, P11DCarCollection] = UNSET
    vans: Union[Unset, P11DVans] = UNSET
    loans: Union[Unset, P11DLoanCollection] = UNSET
    medical: Union[Unset, P11DSingleItem] = UNSET
    relocation: Union[Unset, P11DSingleItem] = UNSET
    services: Union[Unset, P11DSingleItem] = UNSET
    assets_avail: Union[Unset, P11DAssetAvailableCollection] = UNSET
    other: Union[Unset, P11DOther] = UNSET
    exp_paid: Union[Unset, P11DExpenses] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        transferred: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.transferred, Unset):
            transferred = self.transferred.to_dict()

        payments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.payments, Unset):
            payments = self.payments.to_dict()

        vouchers_or_c_cs: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.vouchers_or_c_cs, Unset):
            vouchers_or_c_cs = self.vouchers_or_c_cs.to_dict()

        living_accom: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.living_accom, Unset):
            living_accom = self.living_accom.to_dict()

        mileage_allow: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mileage_allow, Unset):
            mileage_allow = self.mileage_allow.to_dict()

        cars: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cars, Unset):
            cars = self.cars.to_dict()

        vans: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.vans, Unset):
            vans = self.vans.to_dict()

        loans: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.loans, Unset):
            loans = self.loans.to_dict()

        medical: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.medical, Unset):
            medical = self.medical.to_dict()

        relocation: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.relocation, Unset):
            relocation = self.relocation.to_dict()

        services: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.services, Unset):
            services = self.services.to_dict()

        assets_avail: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assets_avail, Unset):
            assets_avail = self.assets_avail.to_dict()

        other: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.other, Unset):
            other = self.other.to_dict()

        exp_paid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.exp_paid, Unset):
            exp_paid = self.exp_paid.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if transferred is not UNSET:
            field_dict["transferred"] = transferred
        if payments is not UNSET:
            field_dict["payments"] = payments
        if vouchers_or_c_cs is not UNSET:
            field_dict["vouchersOrCCs"] = vouchers_or_c_cs
        if living_accom is not UNSET:
            field_dict["livingAccom"] = living_accom
        if mileage_allow is not UNSET:
            field_dict["mileageAllow"] = mileage_allow
        if cars is not UNSET:
            field_dict["cars"] = cars
        if vans is not UNSET:
            field_dict["vans"] = vans
        if loans is not UNSET:
            field_dict["loans"] = loans
        if medical is not UNSET:
            field_dict["medical"] = medical
        if relocation is not UNSET:
            field_dict["relocation"] = relocation
        if services is not UNSET:
            field_dict["services"] = services
        if assets_avail is not UNSET:
            field_dict["assetsAvail"] = assets_avail
        if other is not UNSET:
            field_dict["other"] = other
        if exp_paid is not UNSET:
            field_dict["expPaid"] = exp_paid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, ExbP11DEmployee]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = ExbP11DEmployee.from_dict(_employee)

        _transferred = d.pop("transferred", UNSET)
        transferred: Union[Unset, P11DAssetTransferredCollection]
        if isinstance(_transferred, Unset):
            transferred = UNSET
        else:
            transferred = P11DAssetTransferredCollection.from_dict(_transferred)

        _payments = d.pop("payments", UNSET)
        payments: Union[Unset, P11DPaymentCollection]
        if isinstance(_payments, Unset):
            payments = UNSET
        else:
            payments = P11DPaymentCollection.from_dict(_payments)

        _vouchers_or_c_cs = d.pop("vouchersOrCCs", UNSET)
        vouchers_or_c_cs: Union[Unset, P11DSingleItem]
        if isinstance(_vouchers_or_c_cs, Unset):
            vouchers_or_c_cs = UNSET
        else:
            vouchers_or_c_cs = P11DSingleItem.from_dict(_vouchers_or_c_cs)

        _living_accom = d.pop("livingAccom", UNSET)
        living_accom: Union[Unset, P11DSingleItem]
        if isinstance(_living_accom, Unset):
            living_accom = UNSET
        else:
            living_accom = P11DSingleItem.from_dict(_living_accom)

        _mileage_allow = d.pop("mileageAllow", UNSET)
        mileage_allow: Union[Unset, P11DSingleItem]
        if isinstance(_mileage_allow, Unset):
            mileage_allow = UNSET
        else:
            mileage_allow = P11DSingleItem.from_dict(_mileage_allow)

        _cars = d.pop("cars", UNSET)
        cars: Union[Unset, P11DCarCollection]
        if isinstance(_cars, Unset):
            cars = UNSET
        else:
            cars = P11DCarCollection.from_dict(_cars)

        _vans = d.pop("vans", UNSET)
        vans: Union[Unset, P11DVans]
        if isinstance(_vans, Unset):
            vans = UNSET
        else:
            vans = P11DVans.from_dict(_vans)

        _loans = d.pop("loans", UNSET)
        loans: Union[Unset, P11DLoanCollection]
        if isinstance(_loans, Unset):
            loans = UNSET
        else:
            loans = P11DLoanCollection.from_dict(_loans)

        _medical = d.pop("medical", UNSET)
        medical: Union[Unset, P11DSingleItem]
        if isinstance(_medical, Unset):
            medical = UNSET
        else:
            medical = P11DSingleItem.from_dict(_medical)

        _relocation = d.pop("relocation", UNSET)
        relocation: Union[Unset, P11DSingleItem]
        if isinstance(_relocation, Unset):
            relocation = UNSET
        else:
            relocation = P11DSingleItem.from_dict(_relocation)

        _services = d.pop("services", UNSET)
        services: Union[Unset, P11DSingleItem]
        if isinstance(_services, Unset):
            services = UNSET
        else:
            services = P11DSingleItem.from_dict(_services)

        _assets_avail = d.pop("assetsAvail", UNSET)
        assets_avail: Union[Unset, P11DAssetAvailableCollection]
        if isinstance(_assets_avail, Unset):
            assets_avail = UNSET
        else:
            assets_avail = P11DAssetAvailableCollection.from_dict(_assets_avail)

        _other = d.pop("other", UNSET)
        other: Union[Unset, P11DOther]
        if isinstance(_other, Unset):
            other = UNSET
        else:
            other = P11DOther.from_dict(_other)

        _exp_paid = d.pop("expPaid", UNSET)
        exp_paid: Union[Unset, P11DExpenses]
        if isinstance(_exp_paid, Unset):
            exp_paid = UNSET
        else:
            exp_paid = P11DExpenses.from_dict(_exp_paid)

        exb_p11d = cls(
            employee=employee,
            transferred=transferred,
            payments=payments,
            vouchers_or_c_cs=vouchers_or_c_cs,
            living_accom=living_accom,
            mileage_allow=mileage_allow,
            cars=cars,
            vans=vans,
            loans=loans,
            medical=medical,
            relocation=relocation,
            services=services,
            assets_avail=assets_avail,
            other=other,
            exp_paid=exp_paid,
        )

        return exb_p11d



from enum import Enum


class UserIndustry(str, Enum):
    NOTSPECIFIED = "NotSpecified"
    AGRICULTURE = "Agriculture"
    APPAREL = "Apparel"
    BANKING = "Banking"
    BIOTECHNOLOGY = "Biotechnology"
    CHEMICALS = "Chemicals"
    COMMUNICATION = "Communication"
    CONSTRUCTION = "Construction"
    CONSULTING = "Consulting"
    EDUCATION = "Education"
    ENGINEERING = "Engineering"
    ENTERTAINMENT = "Entertainment"
    ENVIRONMENTAL = "Environmental"
    FINANCE = "Finance"
    FOODANDBEVERAGE = "FoodAndBeverage"
    GOVERNMENT = "Government"
    HEALTHCARE = "Healthcare"
    HOSPITALITY = "Hospitality"
    INSURANCE = "Insurance"
    LEGAL = "Legal"
    MACHINERY = "Machinery"
    MANUFACTURING = "Manufacturing"
    MEDIA = "Media"
    NOTFORPROFIT = "NotForProfit"
    OTHER = "Other"
    RECREATION = "Recreation"
    RETAIL = "Retail"
    SHIPPING = "Shipping"
    TECHNOLOGY = "Technology"
    TELECOMMUNICATIONS = "Telecommunications"
    TRANSPORTATION = "Transportation"
    UTILITIES = "Utilities"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



import httpx


class StaffologyApiException(Exception):
    def __init__(self, *a, **kw):
        self.response = kw.pop("response")
        super().__init__(*a)


def raise_staffology_exception(response: httpx.Response):
    raise StaffologyApiException(response=response)



import datetime
from typing import Any, Dict, Optional, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.national_insurance_calculation_period_values import (
    NationalInsuranceCalculationPeriodValues,
)
from ...models.pay_periods import PayPeriods
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    effective_date: Union[Unset, None, datetime.datetime] = UNSET,
) -> Dict[str, Any]:
    url = "{}/engine/config/{taxYear}/ni/{payPeriod}".format(
        client.base_url, taxYear=tax_year, payPeriod=pay_period
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_period_start: Union[Unset, None, str] = UNSET
    if not isinstance(period_start, Unset):
        json_period_start = period_start.isoformat() if period_start else None

    params["periodStart"] = json_period_start

    json_period_end: Union[Unset, None, str] = UNSET
    if not isinstance(period_end, Unset):
        json_period_end = period_end.isoformat() if period_end else None

    params["periodEnd"] = json_period_end

    json_effective_date: Union[Unset, None, str] = UNSET
    if not isinstance(effective_date, Unset):
        json_effective_date = effective_date.isoformat() if effective_date else None

    params["effectiveDate"] = json_effective_date

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[NationalInsuranceCalculationPeriodValues]:
    if response.status_code == 200:
        response_200 = NationalInsuranceCalculationPeriodValues.from_dict(
            response.json()
        )

        return response_200
    return raise_staffology_exception(response)


def _build_response(
    *, response: httpx.Response
) -> Response[NationalInsuranceCalculationPeriodValues]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    effective_date: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[NationalInsuranceCalculationPeriodValues]:
    """Get National Insurance Thresholds

     Returns the National Insurance thresholds for the given PayPeriod

    Args:
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        effective_date (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[NationalInsuranceCalculationPeriodValues]
    """

    kwargs = _get_kwargs(
        tax_year=tax_year,
        pay_period=pay_period,
        client=client,
        period_start=period_start,
        period_end=period_end,
        effective_date=effective_date,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    effective_date: Union[Unset, None, datetime.datetime] = UNSET,
) -> Optional[NationalInsuranceCalculationPeriodValues]:
    """Get National Insurance Thresholds

     Returns the National Insurance thresholds for the given PayPeriod

    Args:
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        effective_date (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[NationalInsuranceCalculationPeriodValues]
    """

    return sync_detailed(
        tax_year=tax_year,
        pay_period=pay_period,
        client=client,
        period_start=period_start,
        period_end=period_end,
        effective_date=effective_date,
    ).parsed


async def asyncio_detailed(
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    effective_date: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[NationalInsuranceCalculationPeriodValues]:
    """Get National Insurance Thresholds

     Returns the National Insurance thresholds for the given PayPeriod

    Args:
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        effective_date (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[NationalInsuranceCalculationPeriodValues]
    """

    kwargs = _get_kwargs(
        tax_year=tax_year,
        pay_period=pay_period,
        client=client,
        period_start=period_start,
        period_end=period_end,
        effective_date=effective_date,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    effective_date: Union[Unset, None, datetime.datetime] = UNSET,
) -> Optional[NationalInsuranceCalculationPeriodValues]:
    """Get National Insurance Thresholds

     Returns the National Insurance thresholds for the given PayPeriod

    Args:
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        effective_date (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[NationalInsuranceCalculationPeriodValues]
    """

    return (
        await asyncio_detailed(
            tax_year=tax_year,
            pay_period=pay_period,
            client=client,
            period_start=period_start,
            period_end=period_end,
            effective_date=effective_date,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GovTalkError")


@attr.s(auto_attribs=True)
class GovTalkError:
    """
    Attributes:
        raised_by (Union[Unset, None, str]):
        number (Union[Unset, None, str]):
        type (Union[Unset, None, str]):
        text (Union[Unset, None, str]):
        location (Union[Unset, None, str]):
    """

    raised_by: Union[Unset, None, str] = UNSET
    number: Union[Unset, None, str] = UNSET
    type: Union[Unset, None, str] = UNSET
    text: Union[Unset, None, str] = UNSET
    location: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        raised_by = self.raised_by
        number = self.number
        type = self.type
        text = self.text
        location = self.location

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if raised_by is not UNSET:
            field_dict["raisedBy"] = raised_by
        if number is not UNSET:
            field_dict["number"] = number
        if type is not UNSET:
            field_dict["type"] = type
        if text is not UNSET:
            field_dict["text"] = text
        if location is not UNSET:
            field_dict["location"] = location

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        raised_by = d.pop("raisedBy", UNSET)

        number = d.pop("number", UNSET)

        type = d.pop("type", UNSET)

        text = d.pop("text", UNSET)

        location = d.pop("location", UNSET)

        gov_talk_error = cls(
            raised_by=raised_by,
            number=number,
            type=type,
            text=text,
            location=location,
        )

        return gov_talk_error



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsCarFuel")


@attr.s(auto_attribs=True)
class FpsCarFuel:
    """
    Attributes:
        provided (Union[Unset, None, str]):
        cash_equiv (Union[Unset, None, str]):
        withdrawn (Union[Unset, None, str]):
    """

    provided: Union[Unset, None, str] = UNSET
    cash_equiv: Union[Unset, None, str] = UNSET
    withdrawn: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        provided = self.provided
        cash_equiv = self.cash_equiv
        withdrawn = self.withdrawn

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if provided is not UNSET:
            field_dict["provided"] = provided
        if cash_equiv is not UNSET:
            field_dict["cashEquiv"] = cash_equiv
        if withdrawn is not UNSET:
            field_dict["withdrawn"] = withdrawn

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        provided = d.pop("provided", UNSET)

        cash_equiv = d.pop("cashEquiv", UNSET)

        withdrawn = d.pop("withdrawn", UNSET)

        fps_car_fuel = cls(
            provided=provided,
            cash_equiv=cash_equiv,
            withdrawn=withdrawn,
        )

        return fps_car_fuel



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.audit_event_action import AuditEventAction
from ..models.entity_type import EntityType
from ..models.field_modification import FieldModification
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChangeSummary")


@attr.s(auto_attribs=True)
class ChangeSummary:
    """
    Attributes:
        id (Union[Unset, str]):
        audit_id (Union[Unset, str]):
        date (Union[Unset, datetime.date]):
        name (Union[Unset, None, str]):
        action_performed (Union[Unset, AuditEventAction]):
        entity_type (Union[Unset, EntityType]): [readonly] Specifies the entity for which audit logs need to retrieved
        entity_id (Union[Unset, str]): Id of the underlying entity
        employee_unique_id (Union[Unset, str]): Unique id of the employee this operation belongs to
        action_performed_by (Union[Unset, None, str]):
        action_performed_on (Union[Unset, None, str]):
        unique_id (Union[Unset, str]): Unique id of the owner this request belongs to
        parent_event_type (Union[Unset, None, str]):
        modifications (Union[Unset, None, List[FieldModification]]):
    """

    id: Union[Unset, str] = UNSET
    audit_id: Union[Unset, str] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    name: Union[Unset, None, str] = UNSET
    action_performed: Union[Unset, AuditEventAction] = UNSET
    entity_type: Union[Unset, EntityType] = UNSET
    entity_id: Union[Unset, str] = UNSET
    employee_unique_id: Union[Unset, str] = UNSET
    action_performed_by: Union[Unset, None, str] = UNSET
    action_performed_on: Union[Unset, None, str] = UNSET
    unique_id: Union[Unset, str] = UNSET
    parent_event_type: Union[Unset, None, str] = UNSET
    modifications: Union[Unset, None, List[FieldModification]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        audit_id = self.audit_id
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        name = self.name
        action_performed: Union[Unset, str] = UNSET
        if not isinstance(self.action_performed, Unset):
            action_performed = self.action_performed.value

        entity_type: Union[Unset, str] = UNSET
        if not isinstance(self.entity_type, Unset):
            entity_type = self.entity_type.value

        entity_id = self.entity_id
        employee_unique_id = self.employee_unique_id
        action_performed_by = self.action_performed_by
        action_performed_on = self.action_performed_on
        unique_id = self.unique_id
        parent_event_type = self.parent_event_type
        modifications: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.modifications, Unset):
            if self.modifications is None:
                modifications = None
            else:
                modifications = []
                for modifications_item_data in self.modifications:
                    modifications_item = modifications_item_data.to_dict()

                    modifications.append(modifications_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if audit_id is not UNSET:
            field_dict["auditId"] = audit_id
        if date is not UNSET:
            field_dict["date"] = date
        if name is not UNSET:
            field_dict["name"] = name
        if action_performed is not UNSET:
            field_dict["actionPerformed"] = action_performed
        if entity_type is not UNSET:
            field_dict["entityType"] = entity_type
        if entity_id is not UNSET:
            field_dict["entityId"] = entity_id
        if employee_unique_id is not UNSET:
            field_dict["employeeUniqueId"] = employee_unique_id
        if action_performed_by is not UNSET:
            field_dict["actionPerformedBy"] = action_performed_by
        if action_performed_on is not UNSET:
            field_dict["actionPerformedOn"] = action_performed_on
        if unique_id is not UNSET:
            field_dict["uniqueId"] = unique_id
        if parent_event_type is not UNSET:
            field_dict["parentEventType"] = parent_event_type
        if modifications is not UNSET:
            field_dict["modifications"] = modifications

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        audit_id = d.pop("auditId", UNSET)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        name = d.pop("name", UNSET)

        _action_performed = d.pop("actionPerformed", UNSET)
        action_performed: Union[Unset, AuditEventAction]
        if isinstance(_action_performed, Unset):
            action_performed = UNSET
        else:
            action_performed = AuditEventAction(_action_performed)

        _entity_type = d.pop("entityType", UNSET)
        entity_type: Union[Unset, EntityType]
        if isinstance(_entity_type, Unset):
            entity_type = UNSET
        else:
            entity_type = EntityType(_entity_type)

        entity_id = d.pop("entityId", UNSET)

        employee_unique_id = d.pop("employeeUniqueId", UNSET)

        action_performed_by = d.pop("actionPerformedBy", UNSET)

        action_performed_on = d.pop("actionPerformedOn", UNSET)

        unique_id = d.pop("uniqueId", UNSET)

        parent_event_type = d.pop("parentEventType", UNSET)

        modifications = []
        _modifications = d.pop("modifications", UNSET)
        for modifications_item_data in _modifications or []:
            modifications_item = FieldModification.from_dict(modifications_item_data)

            modifications.append(modifications_item)

        change_summary = cls(
            id=id,
            audit_id=audit_id,
            date=date,
            name=name,
            action_performed=action_performed,
            entity_type=entity_type,
            entity_id=entity_id,
            employee_unique_id=employee_unique_id,
            action_performed_by=action_performed_by,
            action_performed_on=action_performed_on,
            unique_id=unique_id,
            parent_event_type=parent_event_type,
            modifications=modifications,
        )

        return change_summary



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.papdis_employee_contact_postal_address import (
    PapdisEmployeeContactPostalAddress,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeContact")


@attr.s(auto_attribs=True)
class PapdisEmployeeContact:
    """
    Attributes:
        postal_address (Union[Unset, PapdisEmployeeContactPostalAddress]):
        email_address (Union[Unset, None, str]): [readonly]
        secondary_email_address (Union[Unset, None, str]): [readonly]
    """

    postal_address: Union[Unset, PapdisEmployeeContactPostalAddress] = UNSET
    email_address: Union[Unset, None, str] = UNSET
    secondary_email_address: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        postal_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.postal_address, Unset):
            postal_address = self.postal_address.to_dict()

        email_address = self.email_address
        secondary_email_address = self.secondary_email_address

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if postal_address is not UNSET:
            field_dict["postalAddress"] = postal_address
        if email_address is not UNSET:
            field_dict["emailAddress"] = email_address
        if secondary_email_address is not UNSET:
            field_dict["secondaryEmailAddress"] = secondary_email_address

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _postal_address = d.pop("postalAddress", UNSET)
        postal_address: Union[Unset, PapdisEmployeeContactPostalAddress]
        if isinstance(_postal_address, Unset):
            postal_address = UNSET
        else:
            postal_address = PapdisEmployeeContactPostalAddress.from_dict(
                _postal_address
            )

        email_address = d.pop("emailAddress", UNSET)

        secondary_email_address = d.pop("secondaryEmailAddress", UNSET)

        papdis_employee_contact = cls(
            postal_address=postal_address,
            email_address=email_address,
            secondary_email_address=secondary_email_address,
        )

        return papdis_employee_contact



from enum import Enum


class LeaveType(str, Enum):
    UNAUTHORISED = "Unauthorised"
    HOLIDAY = "Holiday"
    SICK = "Sick"
    MATERNITY = "Maternity"
    PATERNITY = "Paternity"
    ADOPTION = "Adoption"
    SHAREDPARENTAL = "SharedParental"
    BEREAVEMENT = "Bereavement"
    SHAREDPARENTALADOPTION = "SharedParentalAdoption"
    PATERNITYADOPTION = "PaternityAdoption"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, List, Optional, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.item import Item
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    processor: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/bureau/employers".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["processor"] = processor

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[Item]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[List[Item]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    processor: Union[Unset, None, str] = UNSET,
) -> Response[List[Item]]:
    """List Employers

     Returns a list of Employers.
    This differs from the main Employer list end point in that the metadata included is more relevant to
    Bureau activities
    It also gives you the ability to filter based on the allocated Processor.
    Note: an entry is returned for each PaySchedule. So if the employer has both a Monthly and a Weekly
    PaySchedule then they'll appear twice

    Args:
        processor (Union[Unset, None, str]):

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        client=client,
        processor=processor,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    processor: Union[Unset, None, str] = UNSET,
) -> Optional[List[Item]]:
    """List Employers

     Returns a list of Employers.
    This differs from the main Employer list end point in that the metadata included is more relevant to
    Bureau activities
    It also gives you the ability to filter based on the allocated Processor.
    Note: an entry is returned for each PaySchedule. So if the employer has both a Monthly and a Weekly
    PaySchedule then they'll appear twice

    Args:
        processor (Union[Unset, None, str]):

    Returns:
        Response[List[Item]]
    """

    return sync_detailed(
        client=client,
        processor=processor,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    processor: Union[Unset, None, str] = UNSET,
) -> Response[List[Item]]:
    """List Employers

     Returns a list of Employers.
    This differs from the main Employer list end point in that the metadata included is more relevant to
    Bureau activities
    It also gives you the ability to filter based on the allocated Processor.
    Note: an entry is returned for each PaySchedule. So if the employer has both a Monthly and a Weekly
    PaySchedule then they'll appear twice

    Args:
        processor (Union[Unset, None, str]):

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        client=client,
        processor=processor,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    processor: Union[Unset, None, str] = UNSET,
) -> Optional[List[Item]]:
    """List Employers

     Returns a list of Employers.
    This differs from the main Employer list end point in that the metadata included is more relevant to
    Bureau activities
    It also gives you the ability to filter based on the allocated Processor.
    Note: an entry is returned for each PaySchedule. So if the employer has both a Monthly and a Weekly
    PaySchedule then they'll appear twice

    Args:
        processor (Union[Unset, None, str]):

    Returns:
        Response[List[Item]]
    """

    return (
        await asyncio_detailed(
            client=client,
            processor=processor,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Cis300Declarations")


@attr.s(auto_attribs=True)
class Cis300Declarations:
    """
    Attributes:
        employment_status (Union[Unset, None, str]):
        verification (Union[Unset, None, str]):
        information_correct (Union[Unset, None, str]):
        inactivity (Union[Unset, None, str]):
    """

    employment_status: Union[Unset, None, str] = UNSET
    verification: Union[Unset, None, str] = UNSET
    information_correct: Union[Unset, None, str] = UNSET
    inactivity: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employment_status = self.employment_status
        verification = self.verification
        information_correct = self.information_correct
        inactivity = self.inactivity

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employment_status is not UNSET:
            field_dict["employmentStatus"] = employment_status
        if verification is not UNSET:
            field_dict["verification"] = verification
        if information_correct is not UNSET:
            field_dict["informationCorrect"] = information_correct
        if inactivity is not UNSET:
            field_dict["inactivity"] = inactivity

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employment_status = d.pop("employmentStatus", UNSET)

        verification = d.pop("verification", UNSET)

        information_correct = d.pop("informationCorrect", UNSET)

        inactivity = d.pop("inactivity", UNSET)

        cis_300_declarations = cls(
            employment_status=employment_status,
            verification=verification,
            information_correct=information_correct,
            inactivity=inactivity,
        )

        return cis_300_declarations



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11PayeLine")


@attr.s(auto_attribs=True)
class P11PayeLine:
    """Lines for the Paye Income Tax table in the P11 Detailed report

    Attributes:
        date (Union[Unset, datetime.date]): [readonly]
        period (Union[Unset, None, str]): [readonly]
        month_number (Union[Unset, None, str]): [readonly]
        week_number (Union[Unset, None, str]): [readonly]
        gross_taxable_pay (Union[Unset, float]): [readonly]
        gross_taxable_pay_ytd (Union[Unset, float]): [readonly]
        tax (Union[Unset, float]): [readonly]
        tax_ytd (Union[Unset, float]): [readonly]
        student_loan (Union[Unset, float]): [readonly]
        postgrad_loan (Union[Unset, float]): [readonly]
        tax_code (Union[Unset, None, str]): [readonly]
    """

    date: Union[Unset, datetime.date] = UNSET
    period: Union[Unset, None, str] = UNSET
    month_number: Union[Unset, None, str] = UNSET
    week_number: Union[Unset, None, str] = UNSET
    gross_taxable_pay: Union[Unset, float] = UNSET
    gross_taxable_pay_ytd: Union[Unset, float] = UNSET
    tax: Union[Unset, float] = UNSET
    tax_ytd: Union[Unset, float] = UNSET
    student_loan: Union[Unset, float] = UNSET
    postgrad_loan: Union[Unset, float] = UNSET
    tax_code: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        period = self.period
        month_number = self.month_number
        week_number = self.week_number
        gross_taxable_pay = self.gross_taxable_pay
        gross_taxable_pay_ytd = self.gross_taxable_pay_ytd
        tax = self.tax
        tax_ytd = self.tax_ytd
        student_loan = self.student_loan
        postgrad_loan = self.postgrad_loan
        tax_code = self.tax_code

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if period is not UNSET:
            field_dict["period"] = period
        if month_number is not UNSET:
            field_dict["monthNumber"] = month_number
        if week_number is not UNSET:
            field_dict["weekNumber"] = week_number
        if gross_taxable_pay is not UNSET:
            field_dict["grossTaxablePay"] = gross_taxable_pay
        if gross_taxable_pay_ytd is not UNSET:
            field_dict["grossTaxablePayYTD"] = gross_taxable_pay_ytd
        if tax is not UNSET:
            field_dict["tax"] = tax
        if tax_ytd is not UNSET:
            field_dict["taxYTD"] = tax_ytd
        if student_loan is not UNSET:
            field_dict["studentLoan"] = student_loan
        if postgrad_loan is not UNSET:
            field_dict["postgradLoan"] = postgrad_loan
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        period = d.pop("period", UNSET)

        month_number = d.pop("monthNumber", UNSET)

        week_number = d.pop("weekNumber", UNSET)

        gross_taxable_pay = d.pop("grossTaxablePay", UNSET)

        gross_taxable_pay_ytd = d.pop("grossTaxablePayYTD", UNSET)

        tax = d.pop("tax", UNSET)

        tax_ytd = d.pop("taxYTD", UNSET)

        student_loan = d.pop("studentLoan", UNSET)

        postgrad_loan = d.pop("postgradLoan", UNSET)

        tax_code = d.pop("taxCode", UNSET)

        p11_paye_line = cls(
            date=date,
            period=period,
            month_number=month_number,
            week_number=week_number,
            gross_taxable_pay=gross_taxable_pay,
            gross_taxable_pay_ytd=gross_taxable_pay_ytd,
            tax=tax,
            tax_ytd=tax_ytd,
            student_loan=student_loan,
            postgrad_loan=postgrad_loan,
            tax_code=tax_code,
        )

        return p11_paye_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pdf_password_type import PdfPasswordType
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmailAttachment")


@attr.s(auto_attribs=True)
class EmailAttachment:
    """
    Attributes:
        url (str):
        filename (str):
        pdf_password_type (Union[Unset, PdfPasswordType]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    url: str
    filename: str
    pdf_password_type: Union[Unset, PdfPasswordType] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        filename = self.filename
        pdf_password_type: Union[Unset, str] = UNSET
        if not isinstance(self.pdf_password_type, Unset):
            pdf_password_type = self.pdf_password_type.value

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "url": url,
                "filename": filename,
            }
        )
        if pdf_password_type is not UNSET:
            field_dict["pdfPasswordType"] = pdf_password_type
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        filename = d.pop("filename")

        _pdf_password_type = d.pop("pdfPasswordType", UNSET)
        pdf_password_type: Union[Unset, PdfPasswordType]
        if isinstance(_pdf_password_type, Unset):
            pdf_password_type = UNSET
        else:
            pdf_password_type = PdfPasswordType(_pdf_password_type)

        id = d.pop("id", UNSET)

        email_attachment = cls(
            url=url,
            filename=filename,
            pdf_password_type=pdf_password_type,
            id=id,
        )

        return email_attachment



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="LogEntry")


@attr.s(auto_attribs=True)
class LogEntry:
    """
    Attributes:
        date (Union[Unset, datetime.date]):
        message (Union[Unset, None, str]):
    """

    date: Union[Unset, datetime.date] = UNSET
    message: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        message = self.message

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        message = d.pop("message", UNSET)

        log_entry = cls(
            date=date,
            message=message,
        )

        return log_entry



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="StatutoryPayReportLine")


@attr.s(auto_attribs=True)
class StatutoryPayReportLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        department (Union[Unset, None, str]):
        smp (Union[Unset, float]):
        spp (Union[Unset, float]):
        sap (Union[Unset, float]):
        shpp (Union[Unset, float]):
        spbp (Union[Unset, float]):
        total_stat_pay (Union[Unset, float]):
    """

    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    department: Union[Unset, None, str] = UNSET
    smp: Union[Unset, float] = UNSET
    spp: Union[Unset, float] = UNSET
    sap: Union[Unset, float] = UNSET
    shpp: Union[Unset, float] = UNSET
    spbp: Union[Unset, float] = UNSET
    total_stat_pay: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        department = self.department
        smp = self.smp
        spp = self.spp
        sap = self.sap
        shpp = self.shpp
        spbp = self.spbp
        total_stat_pay = self.total_stat_pay

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if department is not UNSET:
            field_dict["department"] = department
        if smp is not UNSET:
            field_dict["smp"] = smp
        if spp is not UNSET:
            field_dict["spp"] = spp
        if sap is not UNSET:
            field_dict["sap"] = sap
        if shpp is not UNSET:
            field_dict["shpp"] = shpp
        if spbp is not UNSET:
            field_dict["spbp"] = spbp
        if total_stat_pay is not UNSET:
            field_dict["totalStatPay"] = total_stat_pay

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        department = d.pop("department", UNSET)

        smp = d.pop("smp", UNSET)

        spp = d.pop("spp", UNSET)

        sap = d.pop("sap", UNSET)

        shpp = d.pop("shpp", UNSET)

        spbp = d.pop("spbp", UNSET)

        total_stat_pay = d.pop("totalStatPay", UNSET)

        statutory_pay_report_line = cls(
            employee=employee,
            payroll_code=payroll_code,
            department=department,
            smp=smp,
            spp=spp,
            sap=sap,
            shpp=shpp,
            spbp=spbp,
            total_stat_pay=total_stat_pay,
        )

        return statutory_pay_report_line
